// validation constant bits

var VAL_REQUIRED = 1;
var VAL_MAX = 2;
var VAL_EMAIL = 4;
var VAL_DATE = 8;
//var VAL_ = 16;
//var VAL_ = 32;
//var VAL_ = 64;
//var VAL_ = 128;
//var VAL_ = 256;
//var VAL_ = 512;
//var VAL_ = 1024;
//var VAL_ = 2048;
//var VAL_ = 4096;
var VAL_CUSTOM = 8192;

// validation class
validation = {	
	forms  : {},
	
	fieldValue : function(el) {
		return $.trim(el.val());
	},
	validateRequired : function(el, val, rule) {
		if (val==null) return false;		
		return (val!="");
	},
	validateMax : function(el, val, rule) {
		if (val==null) return false;
		if (rule.max==null) return true;
		return (val.length<=rule.max);
	},
	validateEmail : function(el, val, rule) {
		if (val==null) return false;		
		var re = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/ 
		return val.match(re);	
	},
	validateDate : function(el, val, rule) {
		if (val==null) return false;
		return true;
	},
	validateRule : function(el, v, rule) {
		res = true;
		var val = this.fieldValue(el);
		var checks = [];
		if (res && ((rule.checks & VAL_REQUIRED) == VAL_REQUIRED))
		{
			res = this.validateRequired(el, val, rule);
			if (!res)
				checks.push(VAL_REQUIRED);
		}
		
		if (res && ((rule.checks & VAL_MAX) == VAL_MAX))
		{
			res = this.validateMax(el, val, rule);
			if (!res) 
				checks.push(VAL_MAX);
		}
		
		if (res && ((rule.checks & VAL_EMAIL) == VAL_EMAIL))
		{
			res = this.validateEmail(el, val, rule);
			if (!res)
				checks.push(VAL_EMAIL);
		}

		if (res && ((rule.checks & VAL_CUSTOM) == VAL_CUSTOM) && v.validate!=null)
		{
			res = v.validate(el, val);
			if (!res) 
				checks.push(VAL_CUSTOM);
		}
		else
		if (!res && v.validate!=null)
		{
			res = v.validate(el,val);
			if (res)
			{
				if (v.resolvedClass!=null)
					el.removeClass(v.resolvedClass);
				if (v.errorClass!=null)
					el.removeClass(v.errorClass);				
			}
		}
		
		if (!res)
			v.violations.push({ el : el, rule : rule, checks : checks });
		return res;
	},
		
	violationMessage : function (v, violation ){
		var msg = "";
		if (violation.checks.length==0)
			return "";
		if (v.messages!=null)
		{
			$.each(v.messages, function(index, value) {
				if (value.check == violation.checks[0]) {
					msg = value.message;
					if (violation.checks[0] == VAL_MAX && violation.rule.max!=null)
						msg = msg.replace("{max}",violation.rule.max).replace("{name}",violation.rule.name);
				}
			});
		}
		
		if (msg == "" && violation.rule.message!=null)
			msg = violation.rule.message;
		
		return msg;
	},
	
	setHandlers : function(v) {
		var self = this;
		for (var i=0; i<v.rules.length; i++) {
			var rule = v.rules[i];
			$(rule.fields).each(function(index) {
				$(this).off('keyup');
				$(this).off('change');
				$(this).on('keyup', function() {
					self._check(v.name,true);
				});
				$(this).on('change', function() {
					self._check(v.name, true);	
				});
			});		
		}		
	},
	
	update : function(name) {
		v = this.forms[name];
		this.setHandlers(v);				
	},
	
	create : function(name,v) {
		this.forms[name] = v;
		v.name = name;
		this.setHandlers(v);				
	},
		
	_check : function(name, update) {
		var self = this;
		v = this.forms[name];

		if (update) {
			if (v.violations == null)
				return;
			if (v.violations.length == 0)
				return;
		}
				
		if (v.violations && v.violations.length>0) {
			for (var i=0; i<v.violations.length; i++)
			{
				el = v.violations[i].el;
				if (v.resolvedClass!=null)
					el.addClass(v.resolvedClass);
				if (v.errorClass!=null)
					el.removeClass(v.errorClass);
			}
		}
					
		v.violations = [];
		if (v!=null) {
			for (var i=0; i<v.rules.length; i++) {
				var rule = v.rules[i];
				$(rule.fields).each(function(index) {
					self.validateRule($(this), v, rule);
				});		
			}
		}	
		if (v.violations.length>0) {
			var violations_txt = "<ul><li class='intro'><span>We're sorry!</span> We need you to correct or provide the following information:</li>";
			for (var i=0; i<v.violations.length; i++)
			{
				el = v.violations[i].el;
				// focus on first error field
				if (i==0 && !update)
					el.focus();			
				// adjust style of violated field
				if (v.errorClass!=null)
					el.addClass(v.errorClass);
				if (v.resolvedClass!=null)
					el.removeClass(v.resolvedClass);
				violations_txt += "<li>- "+self.violationMessage(v, v.violations[i])+"</li>";
			}
			
			violations_txt += "</ul>";
			
			if (v.setMessage!=null)
				v.setMessage(violations_txt)
				
			return false;
		}
		else {
			if (v.setMessage!=null)
				v.setMessage("");				
			return true;			
		}
				
	},
	checkSuccess : null,
	checkError : null,
	processCheck : function(name) {
		v = this.forms[name];		
		if (this._check(name,false)) {
			if (this.checkSuccess!=null)
				this.checkSuccess();
		}
		else{
			if (this.checkError!=null)
				this.checkError(this.violationMessage(v, v.violations[0]));
		}			
	},
	check : function (name, onSuccess, onError) {
		var self = this;
		if (onSuccess!=null)
			this.checkSuccess = onSuccess;
		if (onError!=null)
			this.checkError = onError;
		
		v = this.forms[name];		
		if (v.prepare!=null)
			v.prepare(function() {
				self.processCheck(name);
			});
		else
			this.processCheck(name);
	}
	
}
		