// -----------------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------
// autocomplete system
// -----------------------------------------------------------------------------------------------------------------------------
// custom autocomplete for requests for operators /requests
// -----------------------------------------------------------------------------------------------------------------------------
operator_requests_autocomplete = {

    $selector : null,

    _search : null,

    selectedItem : null,

    _getRender : function(ul, item) {
        return $( "<li></li>" )
            .data( "item.autocomplete", item )
            .append( "<a>"+ item.label + "</a>" )
            .appendTo( ul );
    },

    _select : function(event,ui) {
        this.selectedItem = ui.item;
        if (this._search)
            this._search(ui.item.value);
    },

    _source : function(request, response) {
        var matcher = new RegExp("\\b" + $.ui.autocomplete.escapeRegex(request.term), "i");
        response($.grep(search_bar_data, function(value) {
            return matcher.test(value.name)
                || matcher.test(value.aliases);
        }).slice(0,6));
    },

    start : function($selector) {

        this.$selector = $selector;
        var self = this;

        var options = {
            source: this._source,
            minLength: 1,
            autoFocus: true,
            delay: 0,
            select: this._select.bind(this),
            change: function (event, ui) {
                if ((self.selectedItem && self.selectedItem.value != self.$selector.val()) || self.$selector.val() === "")
                    self.selectedItem = null;
            }
        }

        $selector.autocomplete(options).
        data("ui-autocomplete").
            _renderItem = this._getRender;

    }

};

// -----------------------------------------------------------------------------------------------------------------------------
// base object with general functionality for top bar autocompletes
//
// this object will be injected into the implementation objects for extension purposes
// -----------------------------------------------------------------------------------------------------------------------------
class autocomplete_base {

    constructor() {

        this.type = "localdata.3";
        this.loading = false;
        this.loadingById = [];
        this.data = null;
        this.dataById = [];
        this.pData = null;
        this.$selector = null;
        this.$autocomplete = null;
        this.abort = false;
        this.processed = false;
        this.matched = [];
        this.result = [];
        this.term = "";
        this.localDebug = "";
        this.localDebugError = false;
        this.startDate = null;

        this.selectedItem = null;

        this.moreDataSupport = false;
        this.moreDataTerms = [];

        // if forceLoad = true, localbrowser cache is being ignored
        this.forceLoad = false;
        this.storageVersion = 1;

    }

    // -- abstract overrides -------------------
    select(ui) {}
    mergeData(data) {}
    setOrder(item, match_type) {}
    text(ui) {}
    resultObject(item, resultObject) {}
    searchData(item, term) {}
    getRender(item) {}
    // -----------------------------------------

    sleep(ms) {
        return new Promise(resolve => setTimeout(resolve, ms));
    }

    onLocalData(data) {
        return data;
    }

    change(val) {

    }

    orderSort(a,b) {
        var o = a.order - b.order;
        if (o === 0) {
            return (""+a.value).length - (""+b.value).length;
        }
        return o;
    }

    // filter localdata with term
    filter(term, response) {

        this.selectedItem = null;

        var self = this;
        self.term = term;

        if (self._onFilter!=null)
            self._onFilter(term);


        // call actual functionality after 0.25 sec of idle state keyup

        var _filter = function (term, response) {

            // this item object will be changed on the fly and added to search result set
            var item = {

                match : "",
                value : null,
                order : 0,

                addResult: function(suborder = 0) {

                    var order = (this.order * 100) + suborder;

                    // default result object
                    var resultObject = {
                        idx : self.result.length,
                        match : this.match,
                        value : this.value,
                        order : order
                    };

                    // [implementation] adjustment ot the result object with data
                    resultObject = self.resultObject(this, resultObject);

                    if (self.matched[resultObject.matchId] == null) {

                        // item was not found yet so add to result set

                        self.matched[resultObject.matchId] = self.result.length;
                        self.result.push(resultObject);
                    }
                    else {

                        // item was found so change if order is lower and has more priority
                        var idx = self.matched[resultObject.matchId];
                        if (order < self.result[idx].order) {
                            resultObject.idx = idx;
                            self.result[idx] = resultObject;
                        }
                    }

                },


                set : function(match_type, value) {
                    this.value = ""+value;
                    this.match = match_type;

                    // [implementation] to set order for this item
                    self.setOrder(this, match_type);
                },

                lowerCaseMatch : function(match_type, value) {
                    value = ""+value;
                    this.set(match_type, value);
                    var pos = this.value.toLowerCase().indexOf(term.toLowerCase());
                    if (pos>=0) {
                        this.addResult(pos);
                    }
                },

                exactMatch : function(match_type, value) {
                    this.set(match_type, value);

                    if (isNumber(term) || !isNaN(parseFloat(""+term))) {
                        if (this.value == parseInt(term))
                            this.addResult();
                    }
                    else {
                        if (this.value == term)
                            this.addResult();
                    }
                },

            };


            var search = function() {

                // search in loaded data

                self.result = [];
                self.matched = [];
                self.processed = false;

                // [implementation] search dataset for results
                self.searchData(item, term);

                var checkProcessed = function() {
                    if (self.processed) {
                        self.result.sort(self.orderSort);

                        if (self.$selector.val().length > 2 && self.result.length === 0 && !self.abort) {
                            // nothing found so check if there is more data
                            if (self.moreDataSupport)
                              self.moreData(term, response);
                        }
                        else
                            response(self.result.splice(0,20));

                    }
                    else {
                        if (!self.abort)
                            window.setTimeout(checkProcessed, 100);
                    }
                }


                if (!self.abort)
                    checkProcessed();

            };

            if (self.loading) {

                // we are still loading the local data so lets wait with searching until the data is loaded

                self.result = [
                    {
                        loading : true
                    }
                ];

                response(self.result);

                // activate the loaded check interval
                var ival = window.setInterval(function() {

                    if (!self.loading && self.data!=null) {

                        window.clearInterval(ival);

                        // data is loaded so search and render response
                        search();
                    }

                }, 100);

            }
            else {

                // data already loaded so lets search

                search();

           }

        }

        var startDelayed = function(s,t,f) {

            self.abort = true;
            window.setTimeout(function() {

                // check if current term is still the same at startDelay call
                if (self.term == t) {

                    self.abort = false;
                    f(t, response);
                }

            },s);

        };

        startDelayed(10, term, _filter);
    }

    moreData(term, response) {

        var self = this;

        // check if we already loaded this period of data
        var _period = storage.get("period/"+self._source);
        if (self.moreDataTerms.indexOf(term)>=0) {
            response([]);
            return; // already searched
        }

        this.loading = true;

        this.result = [
            {
                loading : true
            }
        ];

        response(this.result);


        $sb.jsonCall($sb.url_root + "/" + self._source + "/" + self.type, { "term" : term }, function (data) {

             if (data.term === term) {

                 self.loading = false;
                 self.moreDataTerms.push(term);

                 self.mergeData(self.onLocalData(data));

                 if(self.$selector.is(":focus"))
                     self.$selector.trigger("focus");

             }

        }, null,
            {
                type: "POST",
                error : function( jqXHR, textStatus, errorThrown ) {
                    console.log("more data error!");
                    storage.set("period/" + self._source, self.moreDataPeriod);
                    self.loading = false;
                }
            }
        );
    }

    renderItem(ul, item) {

        var self = this;
        var $el = null;

        if (item.loading) {
            $el = $("<li class='ui-menu-item' style='height:60px; overflow:hidden'></li>")
                .append('<a style="display:none"></a>'+
                    '<div style="padding:10px">'+
                        '<div style="margin-bottom:5px" class="item-head">Loading, please wait...</div>'+
                        '<div class="item-sub">&nbsp;<img src="/images/loader.gif" /></div>'+
                     '</div>')
                .appendTo(ul);
            return $el;
        }

        $el = $("<li></li>")
            // [implementation] item render layout for ul
            .append(self.getRender(item))
            .appendTo(ul);

        return $el;

    }


    start($selector, onSelect = null, type = "localdata.3", pData = null) {

        if (typeof onSelect !== "function" && onSelect != null)
            pData = onSelect;

        if (pData == null)
            pData = {};

        var self = this;

        this.type = type;
        this.$selector = $selector;

        this.localDebug = "";
        this.localDebugError = false;
        this.startDate = new Date();

        if (self._placeHolder != null)
            $selector.attr("placeholder", self._placeHolder);

        var _source = self._source;

        var dataId = self._dataId;
        self.data = self.dataById[dataId+((pData.operator_id)?pData.operator_id:"topheader")];
        self.loading = self.loadingById[dataId+((pData.operator_id)?pData.operator_id:"topheader")];
        self.pData = pData;

        switch(this.type) {

            case "db" :
                // _source will be url that will be called for data
                break;

            default:

                // _source will function and used to process loaded local data
                _source = function (request, response) {

                    self.pData = pData;
                    self.$selector = $selector;

                    self.data = self.dataById[dataId+((pData.operator_id)?pData.operator_id:"topheader")];
                    self.loading = self.loadingById[dataId+((pData.operator_id)?pData.operator_id:"topheader")];

                    self.filter(request.term, response);
                };

                if (this.data == null && !this.loading) {

                    self.loadingById[dataId+((pData.operator_id)?pData.operator_id:"topheader")] = true;

                    if (!this.forceLoad) {

                        if (!pData.operator_id) {

                            // check local browser cache
                            var currentVersion = $selector.data("data-version-" + (self._source.split("autocomplete/")[1]));
                            var localVersion = storage.get("version/" + self._source);
                            var storageVersion = storage.get("_sv/" + self._source);

                            if (localVersion > 0 && localVersion === currentVersion && storageVersion === this.storageVersion) {

                                this.data = storage.get(self._source);
                                if (this.data === false)
                                    this.data = null;
                                else {
                                    this.data = this.onLocalData(this.data);
                                    self.localDebug += "loaded from from browser storage.";
                                    self.loadingById[dataId+((pData.operator_id)?pData.operator_id:"topheader")] = false;
                                }
                            }
                        }
                    }

                    this.pData = pData;
                    this.dataById[dataId+((pData.operator_id)?pData.operator_id:"topheader")] = this.data;

                    if (!this.data) {

                        self.localDebug += ", load-ajax";

                        var _try = 1;

                        var tss = (new Date()).getTime();
                        var getData = function() {

                            var ts = (new Date()).getTime();
                            var tsEnd = function() {

                                var te = (new Date()).getTime();
                                if (te-ts > 1000)
                                    self.localDebug += "<br>in "+numberformat(((te-ts)/1000),2)+" seconds";
                                else
                                    self.localDebug += "<br>in "+(te-ts)+" miliseconds";

                                if (_try>1)
                                    self.localDebug += "<br>total time "+numberformat(((te-tss)/1000),2)+" seconds";

                                _try++;
                            }

                            storage.set("period/" + self._source, "");

                            pData.no_try = _try;

                            $sb.jsonCall($sb.url_root + "/" + self._source + "/" + self.type, pData, function (data) {

                                    if (data.version && !pData.operator_id) {

                                        self.localDebug += ",<br>loaded data to local storage with version "+data.version;

                                        storage.set(self._source, data);
                                        storage.set("_sv/"+self._source, self.storageVersion);
                                        storage.set("version/" + self._source, data.version);
                                    }

                                    self.pData = pData;
                                    self.data = self.onLocalData(data);

                                    self.dataById[dataId+((pData.operator_id)?pData.operator_id:"topheader")] = self.data;
                                    self.loadingById[dataId+((pData.operator_id)?pData.operator_id:"topheader")] = false;

                                    tsEnd();

                                    if(self.$selector.is(":focus"))
                                        self.$selector.trigger("focus");


                                }, null,
                                {
                                    type: "POST",
                                    error : function( jqXHR, textStatus, errorThrown ) {

                                        self.localDebug += "<br>--- try "+(_try)+" : "+((textStatus)?textStatus:"error")+" ["+errorThrown+"] http status "+jqXHR.status;
                                        self.localDebugError = true;
                                        tsEnd();

                                        var _wait = 200;
                                        switch(_try) {
                                            case 2: _wait = 500; break;
                                            case 3: _wait = 500; break;
                                            case 4: _wait = 750; break;
                                            case 5: _wait = 1000; break;
                                        }

                                        if (_try < 6) {

                                            self.localDebug += "<br>RETRY IN " + _wait + "ms";


                                            if (jqXHR.status === 403) {
                                                doRefreshCSRFToken(function(data) {
                                                    self.localDebug += "<br> token success "+data;
                                                    window.setTimeout(getData, _wait);
                                                }, function() {
                                                    self.localDebug += "<br> token fail";
                                                    window.setTimeout(getData, _wait);
                                                });

                                            }
                                            else
                                                window.setTimeout(getData, _wait);
                                        }
                                        else {
                                            var te = (new Date).getMilliseconds();
                                            self.localDebug += "<br> MAX TRIES(5) REACHED! CRITICAL STOP!";
                                        }
                                    }
                                });

                        }

                        getData();

                    }
                }

                break;
        }

        $selector.on("focus", function() {
            $selector.autocomplete("search", $selector.val());
        });

        // initialize jQuery autocomplete class
        $selector.autocomplete(
            {
                source: _source ,
                minLength: 1,
                autoFocus: true,
                delay: 0,

                select: function (event, ui) {

                    self.$selector = $selector;
                    self.pData = pData;

                    self.loading = self.loadingById[dataId+((pData.operator_id)?pData.operator_id:"topheader")];
                    self.data = self.dataById[dataId+((pData.operator_id)?pData.operator_id:"topheader")];

                    self.selectedItem = ui.item;
                    if (onSelect && typeof onSelect === "function") {
                        onSelect(ui.item);
                    }
                    else {
                        self.select(ui);
                    }

                    $selector.val(self.text(ui));
                    return false;
                },

                focus: function (event, ui) {
                    return false;
                },

                change: function (event, ui) {
                    self.change(self.$selector.val());
                },

                create: function (event, ui) {
                }

            }).data("ui-autocomplete")._renderItem = this.renderItem.bind(this);

    }

}

// -----------------------------------------------------------------------------------------------------------------------------

class company_autocomplete extends autocomplete_base {

    // instance specific methods/props ( added to base using extend )
    constructor() {
        super();
        this._source = 'autocomplete/operators';
        this._placeHolder = 'Operator, contact, ID or email';
        this.baseInfo = "";
        this._dataId = "companies";
    }

    // get item text for in text box
    text(ui) {
        return (ui.item.operator)?ui.item.operator.name:ui.item.signup.company_name;
    }

    // item is selected
    select(ui) {

        var op = "";
        if (ui.item.operator!=null)
            op = "" + ui.item.operator["operator_id"];
        else
            op = "o" + ui.item.signup["signup_id"];

        if (this.$selector.selector === "#sign-company-in" || this.$selector.selector === "#sign-company-out") {
            $sb.signinAs(op);
        }
        else {
            if ($sb._op !== op) {

                var dashboard_reset_urls = [
                    $sb.url_root+"/company/edit/"+$sb._op,
                    $sb.url_root+"/safari/edit",
                    $sb.url_root+"/dashboard/tour/",
                ];

                var to_dashboard = false;
                dashboard_reset_urls.forEach(function(url) {
                    if (window.location.href.indexOf(url)===0)
                        to_dashboard = true;
                });

                var toBackend = function(uri) {
                    $sb._op = op;
                    window.location.assign($sb.backend_url(uri));
                }

                if (to_dashboard)
                    toBackend("dashboard");
                else {
                    if (window.location.href.indexOf($sb.url_root+"/companies") === 0)
                        toBackend("companies/"+((op > 1)?"operator":"onboarding"));
                    else {
                        window.location.assign($funcs.windowLocation({
                            "_op": op
                        }, ["pt","sd","ed"]));
                    }
                }

            }
        }

    }

    setDebug () {

        var self = this;

        var $aInfo = $("div.area-operatorselect a.info");

        if (self.baseInfo === "")
            self.baseInfo = $aInfo.data("content");

        var sOperators = "null";
        if (self.data!=null) {
            sOperators = "none";
            if (self.data.operators!=null)
                sOperators = self.data.operators.length;
        }

        var cap = ((self.loading)?"--LOADING":"--LOADED");
        if (self.localDebugError) {
            if (!self.loading)
                cap = "--LOAD SUCCESS WITH AJAX ERRORS";
            else
                cap = "--AJAX LOADING ERROR";
        }

        $aInfo.removeAttr("initialized");
        $aInfo.attr("data-content",
            self.baseInfo +"<br>"+
            self.startDate.toString()+"<br>"+
            "<b><u>Debug Info</u></b><br>"+
            cap+"<br>"+
            "operators : "+sOperators+"<br>"+
            "local : "+self.localDebug
        );

        $aInfo.removeData('content');  // test
        $aInfo.balloon();

    }

    start($selector, onSelect = null, type = "localdata.3", pData = null) {

        super.start($selector, onSelect, type, pData);

        var self = this;
        var selector = this.$selector.selector;
        if (selector === '#top-select-company') {

            $(document).keyup(function(e) {
                if (e.keyCode === 74 && e.ctrlKey && e.altKey)
                    self.setDebug();
            });

        }
    }

    // called when local data is retrieved
    onLocalData(data) {

        if (data.mode) {
            switch(data.mode) {
                case "localdata.3" :

                    var newData = {
                        operators : []
                    };

                    var fields = {
                        i: "operator_id",
                        n: "name",
                        s: "account_status",
                        o: "operators_office_id",
                        e: "emails",
                        c: "country_code"
                    };

                    data.operators.forEach(function(operatorData) {
                        var obj = {};
                        var idx = 0;
                        for (var field in fields) {
                            obj[fields[field]] = operatorData[field];
                        }
                        newData.operators.push(obj);
                    });

                    data = newData;
            }
        }

        return data;

    }

    // add data + unique matching id to result object
    resultObject(item, resultObject) {

        resultObject.operator = item.operator;
        resultObject.signup = item.signup;

        if (item.operator!=null)
            resultObject.matchId = "o_"+item.operator.operator_id;
        else
            resultObject.matchId = "s_"+item.signup.signup_id;

        return resultObject;
    }

    // set item order based on match type
    setOrder(item, match_type) {
        switch(match_type) {
            case "name" : item.order = 1; break;
            case "contact" : item.order = 2; break;
            case "operator_id" : item.order = 3; break;
            case "signup_id" : item.order = 4; break;
            case "email" : item.order = 5; break;
        }
    }

    // search in localdata
    searchData(item, term) {

        var self = this;

        // search operators
        var searchOperator = function(data) {

            item.operator = data.operator;
            item.signup = data.signup;

            if (item.operator!=null) {

                if (isNumber(term) || !isNaN(parseFloat(""+term)))
                    item.lowerCaseMatch("operator_id", item.operator.operator_id);

                if (term.indexOf("p")===0) {
                    item.lowerCaseMatch("operator_id", "p"+item.operator.operator_id);
                }

                item.lowerCaseMatch( "name", item.operator.name);

                for (var n in item.operator.emails) {

                    if (isNaN(parseFloat(""+n))) {
                        item.lowerCaseMatch("contact", n);
                    }

                    item.lowerCaseMatch( "email", item.operator.emails[n]);
                }
            }

            if (item.signup!=null) {

                item.signup.contact = item.signup.firstname+" "+item.signup.lastname;
                item.signup.owner = item.signup.owner_firstname+" "+item.signup.owner_lastname;

                if (isNumber(term) || !isNaN(parseFloat(""+term)))
                    item.lowerCaseMatch("signup_id", item.signup.signup_id);

                if (term.indexOf("o")===0) {
                    item.lowerCaseMatch("signup_id", "o"+item.signup.signup_id);
                }


                item.lowerCaseMatch( "name", item.signup.company_name);
                item.lowerCaseMatch( "contact", item.signup.contact);
                item.lowerCaseMatch( "contact", item.signup.owner);
                item.lowerCaseMatch( "email", item.signup.email);

                if (item.signup.owner_email)
                    item.lowerCaseMatch( "email", item.signup.owner_email);

                if (item.signup.general_email)
                    item.lowerCaseMatch( "email", item.signup.general_email);

                if (item.signup.alternative_email)
                    item.lowerCaseMatch( "email", item.signup.alternative_email);

            }

        }

        var c = 0;
        function handle_operators() {

            for(var i=0; i<self.data.operators.length; i++) {

                if (self.abort) {
                    break;
                }

                var operator = self.data.operators[i];
                searchOperator( {
                    operator : operator,
                    signup : operator.signup
                });

            }

            if (!self.abort && self.data.signups) {

                for (var ii=0; ii<self.data.signups.length; ii++) {

                    if (self.abort)
                        break;

                    var signup = self.data.signups[ii];
                    searchOperator({
                        operator : null,
                        signup : signup
                    });

                }
            }

            self.processed = true;
        }

        handle_operators();

    }

    getRender(item) {

        if (this.abort)
            return "";

        var cssStatus = "other";
        if (item.operator) {
            if (item.operator.account_status === "Active")
                cssStatus = "active";
            if (item.operator.account_status.indexOf("Suspended")>=0)
                cssStatus = "suspended";
        }
        else
            cssStatus = "onboarding";

        var _value = item.value;
        switch(item.match) {
            case "operator_id" : _value = ((_value.indexOf("p")===-1)?"p":"")+_value; break;
            case "signup_id" : _value = ((_value.indexOf("o")===-1)?"o":"")+_value; break;
        }

        var maincountry = "";
        if (item.operator!=null && item.operator.country_code)
            maincountry = ' (<span class="country_code">'+item.operator.country_code+"</span>)";

        var itemHead = '<span class="name">'+_value+'</span>';
        var itemSub = '<div class="icon icon-'+cssStatus+'">&nbsp;</div><span class="name">'+((item.operator)?item.operator.name:item.signup.company_name)+'</span><span class="maincountry">'+maincountry+'</span>';
        if (item.match === "name") {
            itemHead += '<span class="maincountry">'+maincountry+'</span>';
            itemSub = '<div class="icon icon-'+cssStatus+'">&nbsp;</div><span class="account_status">'+((item.operator)?item.operator.account_status:item.signup.status)+'</span>';
        }

        return '<a>'+
            '<div class="item-head">'+itemHead+'</div>'+
            '<div class="item-sub">'+itemSub+'</div>' +
            '</a>';

    }

    startOnSelector(selector) {

        var self = this;
        var $selected = $(selector);

        if($selected.length) {
            window.setTimeout(function() {
                self.start($selected, null, "localdata.3");
            }, 500);
        }
    }

}

// -----------------------------------------------------------------------------------------------------------------------------

class requests_autocomplete extends autocomplete_base {

    constructor() {
        super();
        this._source = 'autocomplete/requests';
        this._placeHolde = 'Ref no, name, email, IP, etc.';
        this._dataId = "requests";

        this.request_q = "";
        this.request_search = "";

    }
    // instance specific methods/props ( added to base using extend )

    start($selector, onSelect = null, type = "localdata.3", pData = null) {

        super.start($selector, onSelect, type, pData);

        var self = this;

        this.moreDataSupport = true;

        var helpContent = ""+
            "<strong>Search quote requests by :</strong>"+
            "<ul class='requests-info'>" +
            "<li><div class='icon icon-client'>&nbsp;</div>Client name</li>"+
            "<li><div class='icon icon-client'>&nbsp;</div>Client email address</li>"+
            "<li><div class='icon icon-nr'>&nbsp;</div>Reference number (e.g. 359172)</li>"+
            "<li><div class='icon icon-client'>&nbsp;</div>User number (e.g. u134724 or 134724)</li>"+
            "<li><div class='icon icon-ip'>&nbsp;</div>IP Address (e.g. 64.281.62.78)</li>"+
            "<li><div class='icon icon-plane'>&nbsp;</div>Tour title</li>"+
            "<li><div class='icon icon-plane'>&nbsp;</div>Tour ID (e.g. t19495 or 19495)</li>"+
            "<li><div class='icon icon-op'>&nbsp;</div>Operator name</li>"+
            "<li><div class='icon icon-op'>&nbsp;</div>Operator ID (e.g. p2510 or 2510)</li>"+
            "</ul>"

        if ($selector.data("adminactions") == 1) {

            $("#adminactions_search_requests_info").attr("data-content", helpContent);

            $("#adminactions_search_requests_button").off("click");
            $("#adminactions_search_requests_button").on("click", function() {
                self.search($("#adminactions_search_requests").val());
            });

        }
        else {

            setCookie("topbar-search-type", "requests");

            var $top_do = $("#top-search-do");
            $top_do.text("Search");
            $top_do.show();

            $top_do.off("click");
            $top_do.on("click", function () {
                self.search($("#top-search-text").val());
            });

            var $top_link = $("#top-search-link");

            $top_link.removeAttr("initialized");
            $top_link.attr("data-content", helpContent);

            $top_link.removeData('content');
            $top_link.balloon();
        }

    }

    change(val) {
        if (val === "") {
            self.request_q = "";
            self.request_search = "";
            self.selectedItem = null;
        }
    }

    // get item text for in text box
    text(ui) {
        return this.request_q;
    }

    requestsPageSearchBox() {
        return (this.$selector.hasClass("requests-search-box"));
    }

    // activate search
    search(val = null, search = "") {
        if (this.requestsPageSearchBox()) {

            var $eid = $("input[name='eid']");

            if (this.request_q!=="") {
                $eid.val(this.request_q);
                $eid.data("search", this.request_search);
            }
            else {
                if (val) {
                    $eid.val(val);
                    $eid.data("search", search);
                }
                else {
                    if ($sb._op > 1) {
                        $eid.val($("#top-select-company").val());
                        $eid.data("search", "p"+$sb._op);
                    }
                }
            }

            window.setTimeout(function() {
                $("#show_requests").trigger("click");
            },100);
        }
        else {
            if (this.request_search!=="")
                window.open($sb.backend_url("admin/requests?q="+encodeURIComponent(this.request_search)));
            else
            if (this.request_q!=="")
                window.open($sb.backend_url("admin/requests?q="+encodeURIComponent(this.request_q)));
            else {
                if (search)
                    window.open($sb.backend_url("admin/requests?q="+encodeURIComponent(search)));
                else
                if (val)
                    window.open($sb.backend_url("admin/requests?q="+encodeURIComponent(val)));
                else {
                    if ($sb._op > 1)
                        window.open($sb.backend_url("admin/requests?q="+encodeURIComponent($("#top-select-company").val())));
                    else
                        window.open($sb.backend_url("admin/requests"));
                }
            }
            this.$selector.trigger("blur");
        }
    }

    // item is selected
    select(ui) {

        this.request_q = "";
        this.request_search = "";

        var item = ui.item;
        var request = item.request;

        var request_field = (item.match === "ref")?"enquiry_id":item.match;

        switch(item.match) {
            case "client" :
            case "client_name" :
            case "client_email" :
                this.request_search = "u"+request["client"];
                this.request_q = request["client_email"];
                if (item.match === "client_name")
                    this.request_q = request["client_name"];
                break;

            case "operator" :
            case "operator_name" :
                this.request_search = "p"+request["operator"];
                this.request_q = request["operator_name"];
                break;

            case "tour" :
            case "tour_title" :
                this.request_search = "t"+request["tour"];
                this.request_q = request["tour_title"];
                break;

            default :
                this.request_q = ""+request[request_field];
                break;
        }

        this.search();
    }

    // add data + unique matching id to result object
    resultObject(item, resultObject) {

        resultObject.request = item.request;

        switch(resultObject.match) {
            case "client" :
            case "client_name" :
            case "client_email" :
                resultObject.matchId = "c_"+item.request.client;
                break;

            case "ref" :
                resultObject.matchId = "r_"+item.request.enquiry_id;
                break;

            case "operator" :
            case "operator_name" :
                resultObject.matchId = "o_"+item.request.operator;
                break;

            case "tour" :
            case "tour_title" :
                resultObject.matchId = "t_"+item.request.tour;
                break;

            case "ip" :
                resultObject.matchId = "i_"+item.request.ip;
                break;
        }
        return resultObject;
    }

    mergeData(data) {
        this.data.requests = this.data.requests.concat(data.requests);
    }

    // called when local data is retrieved
    onLocalData(data) {
        if (data.mode) {
            switch(data.mode) {
                case "localdata.3" :

                    var newData = {
                        requests : []
                    };

                    var fields = {
                        a: "enquiry_id",
                        b: "operator",
                        c: "operator_name",
                        d: "operator_country",
                        e: "client",
                        f: "client_name",
                        g: "client_email",
                        h: "client_country",
                        i: "ip",
                        j: "ip_country",
                        k: "tour",
                        l: "no_of_days",
                        m: "title"
                    };

                    data.requests.forEach(function(requestData) {
                        var obj = {};
                        var idx = 0;
                        for (var field in fields) {
                            obj[fields[field]] = requestData[field];
                        }
                        newData.requests.push(obj);
                    });

                    data = newData;
            }
        }

        data.requests.forEach(function(o) {
            o["tour_search_title_1"] = (o["tour"])?(o["no_of_days"]+"-day "+o["title"]):"";
            o["tour_search_title_2"] = (o["tour"])?(o["no_of_days"]+" day "+o["title"]):"";
            o["tour_title"] = (o["tour"])?(o["no_of_days"]+"-Day "+o["title"]):"";
        });

        return data;
    }

    // set item order based on match type
    setOrder(item, match_type) {
        switch(match_type) {
            case "client_name" : item.order = 1; break;
            case "operator_name" : item.order = 2; break;
            case "operator" : item.order = 3; break;
            case "tour" : item.order = 4; break;
            case "ref" : item.order = 5; break;
            case "client" : item.order = 6; break;
            case "tour_title" : item.order = 7; break;
            case "client_email" : item.order = 8; break;
            case "ip" : item.order = 9; break;
        }
    }

    // search in localdata
    searchData(item, term) {

        var self = this;

        var searchRequest = function(data) {

            item.request = data.request;

            if (item.request!=null) {

                if (isNumber(term) || !isNaN(parseFloat(""+term))) {
                    item.lowerCaseMatch("operator", item.request.operator);
                    item.lowerCaseMatch("client", item.request.client);
                    item.lowerCaseMatch("tour", item.request.tour);
                    item.lowerCaseMatch("ref", item.request.enquiry_id);
                }

                if (term.indexOf("p")===0) {
                    item.lowerCaseMatch("operator", "p"+item.request.operator);
                }
                if (term.indexOf("u")===0) {
                    item.lowerCaseMatch("client", "u"+item.request.client);
                }
                if (term.indexOf("t")===0) {
                    item.lowerCaseMatch("tour", "t"+item.request.tour);
                }

                item.lowerCaseMatch( "client_name", item.request.client_name);
                item.lowerCaseMatch( "client_email", item.request.client_email);

                item.lowerCaseMatch( "tour_title", item.request.tour_search_title_1);
                item.lowerCaseMatch( "tour_title", item.request.tour_search_title_2);
                item.lowerCaseMatch( "operator_name", item.request.operator_name);

                item.lowerCaseMatch( "ip", item.request.ip);

            }

        }

        // search requests

        var c = 0;
        function handle_requests() {

            for (var i=0; i<self.data.requests.length; i++) {

                if (self.abort)
                    break;

                var request = self.data.requests[i];

                searchRequest({
                    request : request
                });

            }

            self.processed = true;

        }

        handle_requests();

    }

    getRender(item) {

        if (this.abort)
            return "";

        var cssIcon = "client";

        var request = item.request;

        var _header = "";
        var _sub = "";



        switch(item.match) {

            case "client_name" :
            case "client_email" :
                _header = request.client_name+" &lt;"+request.client_email+"&gt;";
                _header += '<span class="maincountry"> (<span class="country_code">'+request.client_country+'</span>)</span>';

                _sub = "Client";
                break;

            case "ref" :
                cssIcon = "nr";
                _header = request.enquiry_id;
                _sub = request.client_name+" &lt;"+request.client_email+"&gt;";
                _sub += '<span class="maincountry"> (<span class="country_code">'+request.client_country+'</span>)</span>';
                break;

            case "client" :
                _header = 'User no. u'+request.client;
                _sub = request.client_name+" &lt;"+request.client_email+"&gt;";
                _sub += '<span class="maincountry"> (<span class="country_code">'+request.client_country+'</span>)</span>';
                break;

            case "ip" :
                cssIcon = "ip";
                _header = request.ip;

                if (request.ip_country!=="")
                    _header += '<span class="maincountry"> (<span class="country_code">'+request.ip_country+'</span>)</span>';

                _sub = "IP address";
                break;

            case "operator" :
                cssIcon = "op";
                _header = "Operator p"+request.operator;
                _sub = request.operator_name;
                _sub += '<span class="maincountry"> (<span class="country_code">'+request.operator_country+'</span>)</span>';
                break;

            case "operator_name" :
                cssIcon = "op";
                _header = request.operator_name;
                _header += '<span class="maincountry"> (<span class="country_code">'+request.operator_country+'</span>)</span>';
                _sub = "Tour operator";
                break;

            case "tour" :
                cssIcon = "plane";
                _header = "Tour t"+request.tour;
                _sub = request.tour_title;
                break;

            case "tour_title" :
                cssIcon = "plane";
                _header = request.tour_title;
                _sub = "Tour";
                break;

        }


        var itemHead = '<span class="name">'+_header+'</span>';
        var itemSub = '<div class="icon icon-'+cssIcon+'">&nbsp;</div>'+_sub;

        return '<a>'+
            '<div class="item-head">'+itemHead+'</div>'+
            '<div class="item-sub">'+itemSub+'</div>' +
            '</a>';

    }

}

// -----------------------------------------------------------------------------------------------------------------------------
class reviews_autocomplete extends autocomplete_base {

    constructor() {
        super();

        this._source = 'autocomplete/reviews';
        this._placeHolder = 'ID, name, email, operator, etc.';
        this._dataId = "reviews";

        this.review_q = "";
        this.review_search = "";
    }

    // called when local data is retrieved
    onLocalData(data) {
        if (data.mode) {
            switch(data.mode) {
                case "localdata.3" :

                    var newData = {
                        reviews : []
                    };

                    var fields = {
                        a: "review_id",
                        b: "review_type",
                        c: "operator_id",
                        d: "operator_name",
                        e: "operator_country",
                        f: "country",
                        g: "park",
                        h: "ip",
                        i: "ip_country",
                        j: "reviewer_name",
                        k: "reviewer_email",
                        l: "reviewer_country",
                    };

                    data.reviews.forEach(function(reviewData) {
                        var obj = {};
                        var idx = 0;
                        for (var field in fields) {
                            obj[fields[field]] = reviewData[field];
                        }
                        newData.reviews.push(obj);
                    });

                    data = newData;
            }
        }

        return data;
    }

    change(val) {
        if (val === "") {
            self.review_q = "";
            self.review_search = "";
            self.selectedItem = null;
        }
    }

    text(ui) {
        return this.review_q;
    }

    mergeData(data) {
        this.data.reviews = this.data.reviews.concat(data.reviews);
    }

    start($selector, onSelect = null, type = "localdata.3", pData = null) {

        super.start($selector, onSelect, type, pData);

        var self = this;
        this.moreDataSupport = true;
        var helpContent =
            "<strong>Search reviews by :</strong>" +
            "<ul class='requests-info'>" +
            "<li><div class='icon icon-client'>&nbsp;</div>Reviewer name</li>" +
            "<li><div class='icon icon-client'>&nbsp;</div>Reviewer email address</li>" +
            "<li><div class='icon icon-nr'>&nbsp;</div>ID operator review (e.g. or1234 or 1234)</li>" +
            "<li><div class='icon icon-country'>&nbsp;</div>ID country review (e.g. cr1234 or 1234)</li>" +
            "<li><div class='icon icon-park'>&nbsp;</div>ID park review (e.g. pr1234 or 1234)</li>" +
            "<li><div class='icon icon-op'>&nbsp;</div>Operator ID (e.g. p2510 or 2510)</li>" +
            "<li><div class='icon icon-op'>&nbsp;</div>Operator name</li>" +
            "<li><div class='icon icon-ip'>&nbsp;</div>IP Address (e.g. 64.281.62.78)</li>" +
            "</ul>";

        if ($selector.data("adminactions") == 1) {

            $("#adminactions_search_reviews_info").attr("data-content", helpContent);

            $("#adminactions_search_reviews_button").off("click");
            $("#adminactions_search_reviews_button").on("click", function() {
                self.search($("#adminactions_search_reviews").val());
            });

        }
        else {

            setCookie("topbar-search-type", "reviews");

            var $top_do = $("#top-search-do");
            $top_do.text("Search");
            $top_do.show();

            $top_do.off("click");
            $top_do.on("click", function () {
                self.search($("#top-search-text").val());
            });

            var $top_link = $("#top-search-link");
            $top_link.removeAttr("initialized");
            $top_link.attr("data-content", helpContent);

            $top_link.removeData('content');
            $top_link.balloon();
        }
    }

    reviewsPageSearchBox() {
        return (this.$selector.hasClass("review-search-box"));
    }

    // activate search
    search(val = null, search = "") {
        if (this.reviewsPageSearchBox()) {

            var $eid = $("input[name='review_search_input']");
            if (this.review_q !== "") {
                $eid.val(this.review_q);
                $eid.data("search", this.review_search);
            }
            else {
                if (val) {
                    $eid.val(val);
                    $eid.data("search", search);
                }
                else
                if ($sb._op > 1) {
                    $eid.val($("#top-select-company").val());
                    $eid.data("search", "p"+$sb._op);
                }
            }

            window.setTimeout(function() {
                $("#show_reviews").trigger("click");
            },100);
        }
        else {
            if (this.review_search !== "")
                window.open($sb.backend_url("content/reviews/search/" + encodeURIComponent(this.review_search)));
            else
            if (this.review_q !== "")
                window.open($sb.backend_url("content/reviews/search/" + encodeURIComponent(this.review_q)));
            else {
                if (search)
                    window.open($sb.backend_url("content/reviews/search/" + encodeURIComponent(search)));
                else
                if (val)
                    window.open($sb.backend_url("content/reviews/search/" + encodeURIComponent(val)));
                else {
                    if ($sb._op > 1)
                        window.open($sb.backend_url("content/reviews/search/"+encodeURIComponent($("#top-select-company").val())));
                    else
                        window.open($sb.backend_url("content/reviews/search"));
                }
            }
            this.$selector.trigger("blur");
        }
    }

    // item is selected
    select(ui) {

        this.review_q = "";
        this.review_search = "";

        var item = ui.item;
        var review = item.review;

        var review_field = item.match;

        switch(item.match) {
            case "reviewer_name" :
            case "reviewer_email" :
                this.review_q = review["reviewer_email"];
                break;

            case "operator" :
            case "operator_name" :
                this.review_search = "p"+review["operator_id"];
                this.review_q = review["operator_name"];
                break;

            case "review_operator" : this.review_q = "or"; review_field = "review_id"; break;
            case "review_country" :  this.review_q = "cr"; review_field = "review_id"; break;
            case "review_park" :     this.review_q = "pr"; review_field = "review_id"; break;

            default :
                this.review_q += ""+review[review_field];
                break;
        }

        this.search();

    }

    // add data + unique matching id to result object
    resultObject(item, resultObject) {

        resultObject.review = item.review;
        resultObject.review.ip_country = "NL";

        switch(resultObject.match) {
            case "reviewer_name" :
            case "reviewer_email" :
                resultObject.matchId = "c_"+item.review.reviewer_name;
                break;

            case "review_operator" :
                resultObject.matchId = "or_"+item.review.review_id;
                break;
            case "review_country" :
                resultObject.matchId = "cr_"+item.review.review_id;
                break;
            case "review_park" :
                resultObject.matchId = "pr_"+item.review.review_id;
                break;

            case "operator" :
            case "operator_name" :
                resultObject.matchId = "o_"+item.review.operator_id;
                break;

            case "ip" :
                resultObject.matchId = "i_"+item.review.ip;
                break;
        }

        return resultObject;
    }

    // set item order based on match type
    setOrder(item, match_type) {
        switch(match_type) {
            case "reviewer_name" : item.order = 1; break;
            case "operator_name" : item.order = 2; break;
            case "operator" : item.order = 3; break;
            case "review_operator" : item.order = 4; break;
            case "review_country" : item.order = 5; break;
            case "review_park" : item.order = 6; break;
            case "reviewer_email" : item.order = 7; break;
            case "ip" : item.order = 8; break;
        }
    }

    // search in localdata
    searchData(item, term) {

        var self = this;

        var searchReview = function(data) {

            item.review = data.review;

            if (item.review!=null) {

                if (isNumber(term) || !isNaN(parseFloat(""+term))) {

                    if (item.review.review_type === "operator") {
                        item.lowerCaseMatch("operator", item.review.operator_id);
                        item.lowerCaseMatch("review_operator", item.review.review_id);
                    }

                    if (item.review.review_type === "country")
                        item.lowerCaseMatch("review_country", item.review.review_id);

                    if (item.review.review_type === "park")
                        item.lowerCaseMatch("review_park", item.review.review_id);

                }

                if (term.indexOf("p")===0 && item.review.review_type === "operator") {
                    item.lowerCaseMatch("operator", "p"+item.review.operator_id);
                }

                if (term.indexOf("or")===0 && item.review.review_type === "operator") {
                    item.lowerCaseMatch("review_operator", "or"+item.review.review_id);
                }

                if (term.indexOf("cr")===0 && item.review.review_type === "country") {
                    item.lowerCaseMatch("review_country", "cr"+item.review.review_id);
                }

                if (term.indexOf("pr")===0 && item.review.review_type === "park") {
                    item.lowerCaseMatch("review_park", "pr"+item.review.review_id);
                }

                item.lowerCaseMatch( "reviewer_name", item.review.reviewer_name);
                item.lowerCaseMatch( "reviewer_email", item.review.reviewer_email);

                if (item.review.review_type === "operator") {
                    item.lowerCaseMatch("operator_name", item.review.operator_name);
                }

                item.lowerCaseMatch( "ip", item.review.ip);

            }

        }

        var c = 0;
        function handle_reviews() {

            // search requests
            for (var i=0; i < self.data.reviews.length; i++) {

                if (self.abort)
                    break;

                var review = self.data.reviews[i];

                searchReview({
                    review : review
                });

            };

            self.processed = true;

        }

        handle_reviews();

    }

    getRender(item) {

        if (this.abort)
            return "";

        var cssIcon = "client";

        var review = item.review;

        var _header = "";
        var _sub = "";

        switch(item.match) {

            case "reviewer_name" :
            case "reviewer_email" :
                _header = review.reviewer_name+" &lt;"+review.reviewer_email+"&gt;"+ '<span class="maincountry"> (<span class="country_code">'+review.reviewer_country+'</span>)</span>'
                _sub = "Reviewer";
                break;

            case "review_operator" :
                cssIcon = "nr";
                _header = "Review or"+review.review_id;
                _sub = review.reviewer_name+" &lt;"+review.reviewer_email+"&gt;";
                break;

            case "review_country" :
                cssIcon = "country";
                _header = "Review cr"+review.review_id;
                _sub = review.country;
                break;

            case "review_park" :
                cssIcon = "park";
                _header = "Review pr"+review.review_id;
                _sub = review.park;
                break;

            case "ip" :
                cssIcon = "ip";
                _header = review.ip;
                if (review.ip_country!==null && review.ip_country!=="")
                    _header+=' <span class="maincountry"> (<span class="country_code">'+review.ip_country+'</span>)</span>';
                _sub = "IP address";
                break;

            case "operator" :
                cssIcon = "op";
                _header = "Operator p"+review.operator_id;
                _sub = review.operator_name+ '<span class="maincountry"> (<span class="country_code">'+review.operator_country+'</span>)</span>';
                break;

            case "operator_name" :
                cssIcon = "op";
                _header = review.operator_name+ '<span class="maincountry"> (<span class="country_code">'+review.operator_country+'</span>)</span>';
                _sub = "Tour operator";
                break;

        }


        var itemHead = '<span class="name">'+_header+'</span>';
        var itemSub = '<div class="icon icon-'+cssIcon+'">&nbsp;</div>'+_sub;

        return '<a>'+
            '<div class="item-head">'+itemHead+'</div>'+
            '<div class="item-sub">'+itemSub+'</div>' +
            '</a>';

    }

}

// -----------------------------------------------------------------------------------------------------------------------------

class tours_autocomplete extends autocomplete_base {

    constructor() {
        super();

        this._source = 'autocomplete/tours';
        this._placeHolder = 'Tour title or ID';
        this._dataId = "tours";

        this.tour_q = "";
        this.tour_id = 0;

    }
    // instance specific methods/props ( added to base using extend )


    // called when local data is retrieved
    onLocalData(data) {

        if (data.mode) {
            switch(data.mode) {
                case "localdata.3" :

                    var newData = {
                        tours : []
                    };

                    var fields = {
                        i: "safari_id",
                        s: "status",
                        n: "no_of_days",
                        t: "title"
                    };

                    data.tours.forEach(function(tourData) {
                        var obj = {};
                        var idx = 0;
                        for (var field in fields) {
                            obj[fields[field]] = tourData[field];
                        }
                        newData.tours.push(obj);
                    });

                    data = newData;
            }
        }

        data.tours.forEach(function(o) {
            o["search_title_1"] = (o["no_of_days"]+"-day "+o["title"]);
            o["search_title_2"] = (o["no_of_days"]+" day "+o["title"]);
        });

        return data;
    }

    _onFilter(term) {
        /*
        if (term===this.tour_q && this.tour_q!=="") {
            $("#top-search-do").show();
        }
        else {
            $("#top-search-do").hide();
        }
        */
    }


    text(ui) {
        return this.tour_q;
    }

    start($selector, onSelect = null, type = "localdata.3", pData = null) {

        super.start($selector, onSelect, type, pData);

        var self = this;

        if ($selector.data("adminactions") == 1) {

            setCookie("topbar-search-type", "tours");

            /*

            $top_do.text("Open");

            $top_do.off("click");
            $top_do.on("click", function () {
                self.search();
            });

             */

            var $top_link = $("#top-search-link");

            $top_link.removeAttr("initialized");
            $top_link.attr("data-content",
                "Only live, paused and declined tours are included in this auto-complete. Inactive and draft tours can be found under the tours tab.<br>" +
                "<br>" +
                $top_link.data("content-tours")
            );

            $top_link.removeData('content');
            $top_link.balloon();
        }
        else {
            var $top_do = $("#top-search-do");
            $top_do.hide();
        }

    }

    search() {
        window.open($sb.backend_url("safari/edit/"+this.tour_id));
    }

    // item is selected
    select(ui) {
        this.tour_q = "";
        var item = ui.item;
        var tour = item.tour;

        var review_field = item.match;

        switch(item.match) {
            case "tour" :
                this.tour_q = "t"+tour["safari_id"];
                break;

            case "tour_title" :
                this.tour_q = tour.no_of_days+"-Day "+tour.title;
                break;
        }

        this.tour_id = tour.safari_id;


        $("#top-search-do").show();

        this.search();

    }

    // add data + unique matching id to result object
    resultObject(item, resultObject) {

        resultObject.tour = item.tour;
        resultObject.matchId = "t_"+item.tour.safari_id;

        return resultObject;
    }

    // set item order based on match type
    setOrder(item, match_type) {
        switch(match_type) {
            case "tour_title" : item.order = 1; break;
            case "tour" : item.order = 2; break;
        }
    }

    // search in localdata
    searchData(item, term) {

        var self = this;

        var searchTour = function(data) {

            item.tour = data.tour;

            if (item.tour!=null) {

                if (isNumber(term) || !isNaN(parseFloat(""+term))) {
                    item.lowerCaseMatch("tour", item.tour.safari_id);
                }

                if (term.indexOf("t")===0) {
                    item.lowerCaseMatch("tour", "t"+item.tour.safari_id);
                }

                item.lowerCaseMatch( "tour_title", item.tour.search_title_1);
                item.lowerCaseMatch( "tour_title", item.tour.search_title_2);

            }

        }

        // search requests
        var c = 0;
        function handle_tours() {

            for(var i=0; i<self.data.tours.length; i++) {

                if (self.abort)
                    break;

                var tour = self.data.tours[i];

                searchTour({
                    tour : tour
                });

            }

            self.processed = true;

        }

        handle_tours();

    }

    getRender(item) {

        if (this.abort)
            return "";

        var cssIcon = "plane";

        var tour = item.tour;

        var _header = "";
        var _sub = "";

        var title = tour.no_of_days+"-Day "+tour.title;
        var sStatus = tour.status.capitalize();

        switch(item.match) {

            case "tour_title" :
                _header = title;
                _sub = sStatus+" Tour";
                break;

            case "tour" :
                _header = "Tour t"+tour.safari_id;
                _sub = sStatus+" Tour: "+title;
                break;

        }

        var itemHead = '<span class="name">'+_header+'</span>';
        var itemSub = '<div class="icon icon-tour-'+tour.status+'">&nbsp;</div>'+_sub;

        return '<a>'+
            '<div class="item-head">'+itemHead+'</div>'+
            '<div class="item-sub">'+itemSub+'</div>' +
            '</a>';

    }

}