
const support = {

    "touch": true,
    "touchonly": true,
    "history": true,
    "storage": false,
    "picture_polyfill": false,
    "hdscreen": false,
    "passive": false,
    "intersectionobserver": false,

    "init": function() {

        // Touch
        if (!(('ontouchstart' in window) || window.DocumentTouch && document instanceof DocumentTouch)) {
            $("html").addClass('no-touch'); // No touch in case of device without touch
            support.touch = false;
            support.touchonly = false;
        } else {
            // In case of a 'desktop' touch-device like Windows with touch-screen. Also add the no-touch class for CSS improvements
            var os = support.getOS();
            if (os == 'Windows Phone' || os == 'iOS' || os == 'Android') {
                // Continue as touch device
            } else {
                $("html").addClass('no-touch');
                support.touchonly = false;

            }
        }

        // History API
        var ua = navigator.userAgent;
        if ((ua.indexOf('Android 2.') !== -1 ||
                (ua.indexOf('Android 4.0') !== -1)) &&
            ua.indexOf('Mobile Safari') !== -1 &&
            ua.indexOf('Chrome') === -1 &&
            ua.indexOf('Windows Phone') === -1) {

            support.history = false;

            loadJS("/js/polyfill/history.js", function () {
                support.history = true;
            });

        } else if (!(window.history && 'pushState' in window.history)) {

            support.history = false;

            loadJS("/js/polyfill/history.js", function () {
                support.history = true;
            });
        }

        if (typeof(Storage) !== "undefined") {

            try {
                localStorage.getItem('x');
                support.storage = true;
            } catch (e) {}

        }

        // Test via a getter in the options object to see if the passive property is accessed
        try {
            var opts = Object.defineProperty({}, 'passive', {
                get: function() {
                    support.passive = true;
                }
            });
            window.addEventListener("testPassive", null, opts);
            window.removeEventListener("testPassive", null, opts);
        } catch (e) {}


        support.hdscreen = ((window.matchMedia && (window.matchMedia('only screen and (min-resolution: 124dpi), only screen and (min-resolution: 1.3dppx), only screen and (min-resolution: 48.8dpcm)').matches || window.matchMedia('only screen and (-webkit-min-device-pixel-ratio: 1.3), only screen and (-o-min-device-pixel-ratio: 2.6/2), only screen and (min--moz-device-pixel-ratio: 1.3), only screen and (min-device-pixel-ratio: 1.3)').matches)) || (window.devicePixelRatio && window.devicePixelRatio > 1.3));

        // Polyfill for Object.keys
        if (!Object.keys) Object.keys = function(o) {
            if (o !== Object(o))
                throw new TypeError('Object.keys called on a non-object');
            var k=[],p;
            for (p in o) if (Object.prototype.hasOwnProperty.call(o,p)) k.push(p);
            return k;
        }

        // Polyfill for Array.isArray
        if (!Array.isArray) {
            Array.isArray = function(arg) {
                return Object.prototype.toString.call(arg) === '[object Array]';
            };
        }

        var isIE = support.detectIE();
        if (isIE && isIE <= 12) {
            // Polyfill for picture tag in IE and Edge 12
            loadJS("/js/polyfill/picture.min.js", function () {
                support.picture_polyfill = true;
            });
        }

        if (('IntersectionObserver' in window)) {
            //support.intersectionobserver = true; // removed for now, enable later
        }
    },

    "setDeviceType": function() {

        var old_deviceType = vars.deviceType;

        if ($(window).outerWidth() < 744) {
            vars.deviceType = 'mobile';
        } else if ($(window).outerWidth() < 970) {
            vars.deviceType = 'tablet';
        } else {
            vars.deviceType = 'desktop';
        }

        if (old_deviceType != vars.deviceType) {

            if ($(".floating-request-btn").length && $(".floating-request-btn").is(":visible")) {
                $("body").addClass('has-floating-request-btn');
            } else {
                $("body").removeClass('has-floating-request-btn');
            }

            if (vars.deviceType == 'mobile') {
                $(".list__item[target='_blank']:not(.force-blank)").removeAttr('target');
                $("a[data-pos][target='_blank']:not(.force-blank)").removeAttr('target');
            }

        }

    },

    "getOS": function() {
        var unknown = '-';
        var nAgt = navigator.userAgent;
        var os = unknown;
        var clientStrings = [
            {s:'Windows Phone', r:/(Windows Phone)/},
            {s:'Windows 10', r:/(Windows 10.0|Windows NT 10.0)/},
            {s:'Windows 8.1', r:/(Windows 8.1|Windows NT 6.3)/},
            {s:'Windows 8', r:/(Windows 8|Windows NT 6.2)/},
            {s:'Windows 7', r:/(Windows 7|Windows NT 6.1)/},
            {s:'Windows Vista', r:/Windows NT 6.0/},
            {s:'Windows Server 2003', r:/Windows NT 5.2/},
            {s:'Windows XP', r:/(Windows NT 5.1|Windows XP)/},
            {s:'Windows 2000', r:/(Windows NT 5.0|Windows 2000)/},
            {s:'Windows ME', r:/(Win 9x 4.90|Windows ME)/},
            {s:'Windows 98', r:/(Windows 98|Win98)/},
            {s:'Windows 95', r:/(Windows 95|Win95|Windows_95)/},
            {s:'Windows NT 4.0', r:/(Windows NT 4.0|WinNT4.0|WinNT|Windows NT)/},
            {s:'Windows CE', r:/Windows CE/},
            {s:'Windows 3.11', r:/Win16/},
            {s:'Android', r:/Android/},
            {s:'Open BSD', r:/OpenBSD/},
            {s:'Sun OS', r:/SunOS/},
            {s:'Linux', r:/(Linux|X11)/},
            {s:'iOS', r:/(iPhone|iPad|iPod)/},
            {s:'Mac OS X', r:/Mac OS X/},
            {s:'Mac OS', r:/(MacPPC|MacIntel|Mac_PowerPC|Macintosh)/},
            {s:'QNX', r:/QNX/},
            {s:'UNIX', r:/UNIX/},
            {s:'BeOS', r:/BeOS/},
            {s:'OS/2', r:/OS\/2/},
            {s:'Search Bot', r:/(nuhk|Googlebot|Yammybot|Openbot|Slurp|MSNBot|Ask Jeeves\/Teoma|ia_archiver)/}
        ];
        for (var id in clientStrings) {
            var cs = clientStrings[id];
            if (cs.r.test(nAgt)) {
                os = cs.s;
                break;
            }
        }
        return os;
    },

    "detectIE": function() {
        var ua = window.navigator.userAgent;

        var msie = ua.indexOf('MSIE ');
        if (msie > 0) {
            // IE 10 or older => return version number
            return parseInt(ua.substring(msie + 5, ua.indexOf('.', msie)), 10);
        }

        var trident = ua.indexOf('Trident/');
        if (trident > 0) {
            // IE 11 => return version number
            var rv = ua.indexOf('rv:');
            return parseInt(ua.substring(rv + 3, ua.indexOf('.', rv)), 10);
        }

        var edge = ua.indexOf('Edge/');
        if (edge > 0) {
            // Edge (IE 12+) => return version number
            return parseInt(ua.substring(edge + 5, ua.indexOf('.', edge)), 10);
        }

        // other browser
        return false;
    }

}
support.init();


const storage = {

    "get": function(name, type) {

        if (!support.storage) {

            return false;

        }

        if (null === type || typeof type == 'undefined') {
            type = 'local';
        }

        var return_val = (type == 'local') ? localStorage.getItem(name) : sessionStorage.getItem(name);

        if (null === return_val || false == return_val || return_val === "undefined") {
            return false;
       }

        return JSON.parse(return_val);

    },

    "getAll": function(type) {

        if (!support.storage) {

            return false;

        }

        if (null === type || typeof type == 'undefined') {
            type = 'local';
        }

        return (type == 'local') ? localStorage : sessionStorage;

    },

    "set": function(name, val, type) {

        if (!support.storage) {

            return false;

        }

        if (null === type || typeof type == 'undefined') {
            type = 'local';
        }

        if (type == 'local') {
            localStorage.setItem(name, JSON.stringify(val));
        } else {
            sessionStorage.setItem(name, JSON.stringify(val));
        }

    },

    "remove": function(name, type) {

        if (!support.storage) {

            return false;

        }

        if (null === type || typeof type == 'undefined') {
            type = 'local';
        }

        if (type == 'local') {
            localStorage.removeItem(name);
        } else {
            sessionStorage.removeItem(name);
        }

    }

};

function operatorSelectClick(input) {
    window.setTimeout(function() {
        if (input.selectionStart==input.selectionEnd)
            input.select();
    },100);
}

function delayed(f, tm) {
    if (tm == null)
        tm = 10;
    window.setTimeout(f,10);
}
function delayed100(f) { delayed(f, 100) }
function delayed500(f) { delayed(f, 500) }
function delayed1s(f) { delayed(f, 1000) }
function delayed2s(f) { delayed(f, 2000) }
function delayed5s(f) { delayed(f, 5000) }

function testHtml()
{
    var html = filterHtml(
        "<p><pre><strong>DETAILY ITINERARY:</strong><br />"+
        "<strong>Day 1:Dar Es Salaam Hotel /Mikumi National Park</strong><br />"+
        "Morning at 07:00hrs pick up from the place of staying in Dar Es Salaam hotel and drive to Mikumi National Park. The park shares a<br />"+
        "common boundary with Selous Game Reserve to the South forming a vast track of protected wilderness. Lunch will be at Kikoboga Camp or<br />"+
        "Morongoro Town depends on traffic. In the afternoon we will have game viewing drive in the park around the hippo pools (Kisungura<br />"+
        "Circuit). Most animal can be seen coming down to drink at water holes and the Hippo Pools from time to time but the stars of the<br />"+
        "show will always be the hippos themselves. Dinner and overnight at will be at the Mikumi Bush Camp /Tanapa Cottage (En Suite) or Vuma<br />"+
        "Hill Tented Lodge. <em><strong>Meals Plan: Lunch & Dinner</strong></em><br />"+
        "<br /><strong>Day 2: Mikumi National Park/Dar es Salaam</strong><br />"+
        "After an early morning breakfast we will go for another game viewing drive in the park, common animal to be seen are giraffes, elephants,<br />"+
        "buffaloes, impalas, lion, elands, wildebeest, zebras and troops of baboons entertaining to watch and are common in the park. The lucky<br />"+
        "visitor may see the occasional thrilling view of a leopard or sable antelopes with their impressive sweeping horns are sometimes<br />"+
        "glimpsed in the woodland. Wild dogs are an endangered species found in the park from time to time although sightings of them are<br />"+
        "extremely rare and rewarding. In the afternoon drive back to Dar es Salaam, where you will arrive late evening and drop off at the place<br />"+
        "of staying or airport. <strong><em>Meals Plan; Breakfast & Lunch</em></strong><br />"+
        "<br /><strong>End of the safari services</strong><br /><br />"+
        "<br />"+
        "<strong>What included: -</strong> Pick /drop ex hotel before and after the trip - Park entrance fees into the game reserve, accommodation and meals as per<br />"+
        "itinerary, game drives and activities as per itinerary, the services of our knowledgeable English speaking driver/guide, safari transport<br />"+
        "by 4 x 4 (6 seat) with a roof hatch that pop up to enable easy game viewing and photography, Group size is 5 participants on private.<br />"+
        "<br /><strong>Not included in the cost:-</strong> items of a personal nature e.g. laundry, telephone bills, soft drinks, alcoholic and other beverages,<br />"+
        "optional activities not outlined on the itinerary, local/international flights, airport departure tax, visas,<br />"+
        "travel/flying doctors rescue services, tips to the staff (highly recommended ), personal and baggage insurance.</pre></p> <br />");

    alert(html);

}

function doRefreshCSRFToken(onsuccess, onfail) {

    if ($(".signasoperator").length) {
        console.log('no refresh token');
    } else {

        $.post('/refreshtoken', function (data) {

            if (data != '') {
                $('meta[name=csrf-token]').attr('content', data);
                if ($('input[name=_token]').length) {
                    $('input[name=_token]').val(data);
                }
                if (typeof token != 'undefined') {
                    token = data;
                    $.ajaxSetup({
                        headers: {
                            'X-CSRF-TOKEN': token
                        }
                    });
                }
            }

            if (onsuccess)
                onsuccess(data);

        }).fail(function () {
            if (onfail)
                onfail();
        });

    }

}

function refreshCSRFToken() {
    if ($('meta[name=token_refresh]').length) {
        setTimeout(function() {
            finalRefreshCSRFToken();
        }, ($('meta[name=token_refresh]').attr('content') * 1000));
    }
}

function finalRefreshCSRFToken() {
    doRefreshCSRFToken(
        function(){
            refreshCSRFToken();
        },
        function() {
            setTimeout(function() {
                refreshCSRFToken();
            }, (60000 * 5)); // Retry after 5 minutes
        }
    );
}


window.setTimeout(function() {

    var $reooo = $(".reactivation-end-ooo");
    if ($reooo.length) {
        $reooo.off("click");
        $reooo.on("click", function() {

            $('#reveal_modal_center').foundation('reveal', 'close');

            var $oo = $("#outofoffice-notice");
            $oo.html("<img src='/images/load.gif' class='loading' /> Ending Out of Office...");
            $oo.show();

            $.post("/account/out-of-office",
                { 'delete': 1 }
            ).done(function () {
                $reooo.remove();
                $oo
                    .removeClass('warning')
                    .addClass('success')
                    .html('<span class="warning_main_title">' +
                        '<img src="/images/billing/check-success.png" border="0"> ' +
                        'Your \'Out of Office\' has been turned off' +
                        '</span><br><br>'+
                        '<a href="'+$sb.backend_url("dashboard")+'" class="btn btn--orange btn--autowidth continue">Open Tour Operator Admin</a>');
            });
        });
    }
}, 500);

function modal_events($modal) {

    if ($modal.find("#reactivation_result").length) {

        var $modal_center = $("#reveal_modal_center");
        var $reactivation_close = $(".reactivation-close");
        var $reactivation_end_ooo = $(".reactivation-end-ooo");

        var handleClose = function() {
            handleCloseRevealModal();
            $modal_center.removeClass("small-reveal-modal");
        };

        var closeModalCenter = function() {
            $modal_center.foundation('reveal', 'close');
            window.setTimeout(handleClose,200);
        };

        var endOutOfOffice = function() {
            $(this).addClass("loading");
            $.post("/account/out-of-office",
                { 'delete': 1 }
            ).done(function () {
                var $oo = $("#outofoffice-notice");
                closeModalCenter();
                $oo.delay(500).slideUp(500, function() {
                    $oo.removeClass("warning");
                    $oo.addClass("success");
                    $oo.html("<b>Your 'Out of Office' has been turned off</b>");
                    $oo.slideDown(500);
                });
            });

        };

        $reactivation_close.off("click");
        $reactivation_close.on("click", closeModalCenter) ;

        $reactivation_end_ooo.off("click");
        $reactivation_end_ooo.on("click", endOutOfOffice);
    }
}

// get modal html to reveal and if present displays it in #reveal_modal_center
function processModalReveal() {

    var $reveal_modal_center = $("#reveal_modal_center");

    if ($reveal_modal_center.html()!="") {

        $reveal_modal_center.addClass("small-reveal-modal");
        $reveal_modal_center.foundation('reveal', 'open', {
            animation: 'fade'
        });

        $reveal_modal_center.off('opened.fndtn.reveal');
        $reveal_modal_center.on('opened.fndtn.reveal', function () {
            handleOpenRevealModal();
            modal_events($reveal_modal_center);
        });

        $reveal_modal_center.off('closed.fndtn.reveal');
        $reveal_modal_center.on('closed.fndtn.reveal', function () {
            handleCloseRevealModal();
            $reveal_modal_center.removeClass("small-reveal-modal");
        });
    };
}

var isNumber = function isNumber(value)
{
    return typeof value === 'number' && isFinite(value);
}

// --------------------------------------------------------------------------------------------------------------------------------------------------------

function filterHtml(html)
{
    // remove html comments
    html = html.replace(/<!--(.|\n)*?-->/g,"");
    // <br> to <br />
    html = html.replace(/<\s*br.*?>/gi,"<br />");
    // <p *> to <br><br>
    html = html.replace(/<\s*p.*?>/gi,"<br /><br />");
    // <ul *> to <br><br>
    html = html.replace(/<\s*ul.*?>/gi,"<br /><br />");
    // <ol *> to <br><br>
    html = html.replace(/<\s*ol.*?>/gi,"<br /><br />");
    // </h1> to <br><br>
    html = html.replace(/<\/\s*h1.*?>/gi,"<br /><br />");
    // </h2> to <br><br>
    html = html.replace(/<\/\s*h2.*?>/gi,"<br /><br />");
    // </h3> to <br><br>
    html = html.replace(/<\/\s*h3.*?>/gi,"<br /><br />");
    // <li *> to <br>-
    html = html.replace(/<\s*li.*?>/gi,"<br />- ");
    // codes to keep to special codes
    html = html.replace(/<\s*br.*? \/>/gi,"{@br}").replace(/<\s*strong.*?>/gi,"{@b}").replace(/<\s*b?>/gi,"{@b}").replace(/<\s*em.*?>/gi,"{@i}").replace(/<\s*i.*?>/gi,"{@i}");
    html = html.replace(/<\/\s*strong.*?>/gi,"{@/b}").replace(/<\/\s*b?>/gi,"{@/b}").replace(/<\/\s*em.*?>/gi,"{@/i}").replace(/<\/\s*i.*?>/gi,"{@/i}");
    // remove all html tags
    html = html.replace(/<(.|\n)*?>/g,"");
    // replace special codes back
    html = html.replace(/{@\s*br.*?}/gi,"<br />").replace(/{@\s*b.*?}/gi,"<b>").replace(/{@\s*i.*?}/gi,"<i>").replace(/{@\/\s*b.*?}/gi,"</b>").replace(/{@\/\s*i.*?}/gi,"</i>");
    // trim leading and trailing br's
    html = html.trim();
    while (html.indexOf("<br />")==0)
        html = html.substr(6, html.length-6);
    return html;
}

$(function() {

    window.sbalert = window.alert;
    window.alert = function(title_or_message, message_or_callback, callback) {
        if (typeof(title_or_message) == 'undefined') { return false; }

        var title = '', msg = '', cb = '';

        if (typeof(callback) != 'undefined') {
            title = title_or_message;
            msg = message_or_callback;
            cb = callback;
        }
        else
        if (typeof(message_or_callback) != 'undefined') {
            if ($.isFunction(message_or_callback)) {
                cb = message_or_callback;
                msg = title_or_message;
            }
            else {
                title = title_or_message;
                msg = message_or_callback;
            }
        } else {
            msg = title_or_message;
        }

        $(".alert-layer").remove();
        var alert_box = '<div class="alert-layer" "><div>';
        if (title != '') {
            alert_box += '<h1>' + title + '</h1>';
        }
        alert_box += msg;
        alert_box += '<center><input type="submit" value="OK" class="sb-button"></center></div></div>';
        $("body").append(alert_box).css('overflow', 'hidden');
        $(".alert-layer").show().css('opacity', '0.01');
        $(".alert-layer > div").css({'max-height': '650px', 'overflow-y': 'auto'});
        $(".alert-layer > div").css({'margin-top': parseInt($(".alert-layer > div").outerHeight() / 2) * -1, 'margin-left': parseInt($(".alert-layer > div").outerWidth() / 2) * -1});
        $(".alert-layer").fadeTo('fast', 1, function() {
            $(".alert-layer").addClass('finished');
        });
        $(".alert-layer input:eq(0)").click(function() {
            $('.alert-layer').fadeOut('fast', function() { $('.alert-layer').remove(); $("body").css('overflow', 'auto'); });
            if ($.isFunction(cb)) {
                cb();
            }
        });
    };

    var _confirm = window.confirm;
    window.confirm = function(title_or_message, message_or_callback, callback, ok_button, third_button, cancel_button, options) {

        if (typeof(title_or_message) == 'undefined') { return false; }

        if (typeof(ok_button) == 'undefined') { ok_button = 'OK' }
        if (typeof(cancel_button) == 'undefined') { cancel_button = 'Cancel' }

        var title = '', msg = '', cb = '';

        if (typeof(callback) != 'undefined' && typeof(callback) != 'string') {
            if (title_or_message !== false)
                title = title_or_message;
            msg = message_or_callback;
            cb = callback;
        } else if (typeof(message_or_callback) != 'undefined') {
            msg = title_or_message;
            cb = message_or_callback;

            if (typeof(callback) == "string") {
                options = cancel_button;
                cancel_button = third_button;
                third_button = ok_button;
                ok_button = callback;
            }

        } else {
            return _confirm(title_or_message);
        }

        if (!$.isFunction(cb)) {
            return _confirm(title_or_message);
        }

        var _options = {
            third_button_class : "grey"
        }
        if (options != null) {
            for (n in options)
                _options[n] = options[n];
        }

        $(".confirm-layer").remove();
        var confirm_box = '<div class="confirm-layer"><div>';
        if (title != '') {
            confirm_box += '<h1>' + title + '</h1>';
        }
        confirm_box += msg;
        confirm_box += '<br /><center><input type="submit" value="' + ok_button + '" class="sb-button"> ';
        if (typeof(third_button) != 'undefined') {
            confirm_box += ' <input type="submit" value="' + third_button + '" class="sb-button '+_options.third_button_class+'"> ';
        }
        confirm_box += '<input type="submit" value="' + cancel_button + '" class="sb-button grey"></center></div></div>';
        $("body").append(confirm_box).css('overflow', 'hidden');
        $(".confirm-layer").show().css('opacity', '0.01');
        $(".confirm-layer > div").css({'margin-top': parseInt($(".confirm-layer > div").outerHeight() / 2) * -1, 'margin-left': parseInt($(".confirm-layer > div").outerWidth() / 2) * -1});
        $(".confirm-layer").fadeTo('fast', 1);
        $(".confirm-layer input:eq(0)").click(function() {
            if (!$(this).hasClass('loading')) {
                if ($.isFunction(cb)) {
                    $(".confirm-layer input:eq(0)").addClass('loading');
                    var return_val = cb(true);
                    if (typeof return_val == 'undefined' || return_val) {
                        closeConfirmBox();
                    }
                } else {
                    closeConfirmBox();
                }
                return true;
            } else {
                return false;
            }
        });
        if (typeof(third_button) != 'undefined') {
            $(".confirm-layer input:eq(1)").click(function() {
                if (!$(this).hasClass('loading')) {
                    if ($.isFunction(cb)) {
                        $(".confirm-layer input:eq(0)").addClass('loading');
                        var return_val = cb(2);
                        if (typeof return_val == 'undefined' || return_val) {
                            closeConfirmBox();
                        }
                    } else {
                        closeConfirmBox();
                    }
                    return 2;
                } else {
                    return false;
                }
            });
            $(".confirm-layer input:eq(2)").click(function () {
                if ($.isFunction(cb)) {
                    cb(false);
                }
                closeConfirmBox();
                return false;
            });
        } else {
            $(".confirm-layer input:eq(1)").click(function () {
                if ($.isFunction(cb)) {
                    cb(false);
                }
                closeConfirmBox();
                return false;
            });
        }
    }

    // Track basic JavaScript errors
    if (window.addEventListener) {
        window.addEventListener('error', function(e) {
            if (typeof(ga) == "function") {
                ga( 'send',
                    'event',
                    'JavaScript Error',
                    e.message,
                    e.filename + ':  ' + e.lineno,
                    { 'nonInteraction': 1 }
                );
            }
        });
    } else {
        window.attachEvent('onerror', function(e) {
            if (typeof(ga) == "function") {
                ga( 'send',
                    'event',
                    'JavaScript Error',
                    e.message,
                    e.filename + ':  ' + e.lineno,
                    { 'nonInteraction': 1 }
                );
            }
        });
    }

    // Track AJAX errors (jQuery API)
    $(document).ajaxError(function(e, request, settings, exception) {
        if (typeof(ga) == "function") {
            ga( 'send',
                'event',
                'Ajax error',
                settings.url,
                e.result + ' | ' + request.status + ' | ' + exception,
                { 'nonInteraction': 1 }
            );
        }
    });

    $("input[type=button], input[type=submit]").on("click", function() {
        if($(this).hasClass("loading")) {
            return false;
        } else {
            if (!$(this).hasClass('no-loading')) {
                $(this).addClass("loading");
            }
        }
    });

    $(".widget-form textarea").click(function() {
        $(this).select();
    });

    // init top bar (and other) company autocompletes

    var company_top = new company_autocomplete();
    company_top.startOnSelector("#top-select-company");

    window.setTimeout(function() {

        var company_signin = new company_autocomplete();
        company_signin.startOnSelector("#sign-company-in");

        var company_signout = new company_autocomplete();
        company_signout.startOnSelector("#sign-company-out");

    }, 1000);

    // check for a top header search box
    $top_search_text = $("#top-search-text");
    if($top_search_text.length) {

        var top_requests = null;
        var top_reviews = null;
        var top_tours = null;

        // search items autocomplete handler
        var search_elements_init_autocomplete = function() {

            switch($("#search-elements").val()) {
                case "requests" :
                    if (top_requests == null)
                        top_requests = new requests_autocomplete();
                    top_requests.start($top_search_text, null, "localdata.3");
                    break;
                case "reviews" :
                    if (top_reviews == null)
                        top_reviews = new reviews_autocomplete();
                    top_reviews.start($top_search_text, null, "localdata.3");
                    break;
                case "tours" :
                    if (top_tours == null)
                        top_tours = new tours_autocomplete();
                    top_tours.start($top_search_text, null, "localdata.3");
                    break;
            }
        };

        $search_elements = $("#search-elements");
        $searchElementValue = getCookie("topbar-search-type");

        if ($searchElementValue === "")
            $searchElementValue = "requests";

        $search_elements.val($searchElementValue);

        $search_elements.on("change",search_elements_init_autocomplete);
        $search_elements.trigger("change");

    }

    // request search box on /admin/requests page
    $requests_search_box = $("input.requests-search-box");
    if ($requests_search_box.length) {

        requestsData.admin_autocomplete = new requests_autocomplete()
        requestsData.admin_autocomplete.start($requests_search_box, null, "localdata.3");

        $requests_search_box.on("keydown", function (e) {
            if (e.keyCode === 13) {
                $("#show_requests").trigger("click");
                return false;
            }
        });
    }

    $requests_search_box = $("#adminactions_search_requests");
    if ($sb._op > 1 && $requests_search_box.length) {
        // request search box on /companies/operator page
        delayed(function() {
            (new requests_autocomplete()).start($requests_search_box, { operator_id : $sb._op });
            (new reviews_autocomplete()).start($("#adminactions_search_reviews"), { operator_id : $sb._op });
            (new tours_autocomplete()).start($("#adminactions_search_tours"), { operator_id : $sb._op });
        });
    }



    $tiny_mce = $('textarea.tinymce');
    if($tiny_mce.length > 0) {
        $tiny_mce.each(function() {
            var buttons = 'bold,italic';
            if ($(this).hasClass('withlists')) {
                buttons = 'bold,italic,bullist,numlist,outdent,indent,link';
            }
            $(this).tinymce({
                // Location of TinyMCE script
                script_url : '/libs/tiny_mce/tiny_mce.js',

                height: "300",
                // General options
                theme : "advanced",
                plugins : "paste,legacyoutput",
                invalid_elements: "span,img,h1,h2,h3,h4,h5,div,u,table,tr,td,iframe,form,center,hr,input,object,script,textarea,select,p,pre,style,xml,blockquote",
                force_br_newlines : true,
                force_p_newlines : false,
                forced_root_block : false,
                convert_urls: false,
                paste_auto_cleanup_on_paste : true,
                paste_remove_styles : true,
                paste_remove_styles_if_webkit : true,
                paste_strip_class_attributes : true,
                paste_preprocess : function(plugin, args) {
                    args.content = filterHtml(args.content);
                },
                paste_postprocess : function(pl, o) {
                    // Content DOM node containing the DOM structure of the clipboard
                    o.node.innerHTML = filterHtml(o.node.innerHTML);
                },
                // Theme options
                theme_advanced_buttons1 : buttons,
                theme_advanced_buttons2 : "",
                theme_advanced_buttons3 : "",
                theme_advanced_buttons4 : "",
                theme_advanced_toolbar_location : "top",
                theme_advanced_toolbar_align : "left",
                theme_advanced_statusbar_location : "",
                theme_advanced_resizing : true,

                // Example content CSS (should be your site CSS)
                content_css : "/css/AfricaGoTrips.css,https://fonts.googleapis.com/css?family=Noto+Sans",

                // Drop lists for link/image/media/template dialogs
                template_external_list_url : "/libs/tiny_mce/lists/template_list.js",
                external_link_list_url : "/libs/tiny_mce/lists/link_list.js",
                external_image_list_url : "/libs/tiny_mce/lists/image_list.js",
                media_external_list_url : "/libs/tiny_mce/lists/media_list.js",

                onchange_callback : "tinyMCEChanged"
            });
        });
    }

    if (typeof($.browser) != 'undefined' && $.browser) {
        if (($.browser.msie && $.browser.versionNumber <= 8) || ($.browser.mozilla && $.browser.versionNumber <= 10)) {
            var browser_msg = '<div class="unsupportedbrowser-bar"><b>Your internet browser is not supported.</b> Please update your browser: &nbsp;';
            browser_msg += '<a href="https://www.google.com/chrome/?hl=en" target="_blank">Chrome</a> - ';
            browser_msg += '<a href="https://www.mozilla.com/firefox/" target="_blank">Firefox</a> - ';
            browser_msg += '<a href="https://www.microsoft.com/windows/internet-explorer/default.aspx" target="_blank">Internet Explorer</a> - ';
            browser_msg += '<a href="https://www.apple.com/safari/download/" target="_blank">Safari</a>';
            browser_msg += '</div>';
            $(".page-title-container").prepend(browser_msg);
        }
    }

    $("a.btn-close").on("click", function() {
        if($(this).parent().hasClass("alert-box")) {

            key = $(this).data("setconfig");
            if (key!=null) {
                // because we got a set config data element we need to set the current operator config key value to 1
                $funcs.jsonCall("/billing/setconfig", { key : key }, function(data){
                });
            }

            // aanpassing martijnc om beter en gerichter te kunnen closen
            if ($(this).data("close") != "")
            {
                $(this).parents($(this).data("close")).slideUp("slow", function(){
                    $(this).remove();
                });

                return false;
            }

            //$(this).parents(".row-container").slideUp("slow", function() { $(this).remove(); });
            $(this).parent().parent().parent().slideUp("slow", function() { $(this).remove(); });

        }
        return false;
    });

    $('body').mousedown(function(e){
        if ((!$(e.target).parents('.problem-box-holder').length) && (!$(e.target).parents('.history-box-holder').length) && (!$(e.target).parents('.reveal-modal').length)) {

            if ($(".problem-box").hasClass('open')) {
                $(".problem-box .open-button").trigger('click');
            }
            if ($(".history-box").hasClass('open')) {
                $(".history-box .open-button").trigger('click');
            }
            if ($(".email-box").hasClass('open')) {
                closeEmailBox();
            }
        }
    });

    $('body').click(function(e){

        if ((!$(e.target).parents('.alert-layer').length)) {
            if ($(".alert-layer").length && $(".alert-layer").is(":visible") && $(".alert-layer").hasClass('finished')) {
                $(".alert-layer input:eq(0)").trigger('click');
            } else if ((!$(e.target).parents('.modal-box').length)) {
                if ($(".modal-box").is(":visible") && $(".modal-box .remodal-cancel").is(":visible")) {
                    $(".modal-box .remodal-cancel").trigger('click');
                } else if ($(".modal-box").is(":visible") && $(".modal-box .billing-cancel").is(":visible")) {
                    $(".modal-box .billing-cancel").trigger('click');
                }
            }
        }
        if ($(".date-range-picker-holder.hasDatepicker").length && $(".date-range-picker-holder.hasDatepicker").is(":visible") && !$(e.target).parents('.ui-datepicker-group').length) {
            if ( $("#date-input1").val() != '' && $("#date-input2").val() == '' ) {
                $("#date-input1").val('');
                $(".date-range-picker-holder td.dp-highlight").removeClass('dp-highlight');
            }
        }
        if (typeof availability_showing != 'undefined' && $(".load-layer").length && $(".load-layer").is(":visible") && $(".availability-popup").length && $(".availability-popup").is(":visible") && !availability_showing) {
            if (!$(e.target).parents('.availability-popup').length && !$(e.target).hasClass('availability-popup')) {
                hideAvailabilityPopup();
            }
        }
        if ($('body').hasClass('availability-tool') && $(".load-layer").length && $(".load-layer").is(":visible") && $(".selection-popup").length && $(".selection-popup").is(":visible") && !availabilitytool.showingSelection) {
            if (!$(e.target).parents('.selection-popup').length && !$(e.target).hasClass('selection-popup')) {
                availabilitytool.hideSelectionList();
            }
        }
    });

    $(document).on('keyup', function(e) {
        if (e.keyCode === 27) {
            if (($('.percentages-reveal-modal').is(":visible") && !$(e.target).parents('.percentages-reveal-modal').length && !$(e.target).hasClass('percentages-reveal-modal'))) {
                closePercentagesWithConfirm();
                return false;
            }
        }
    });

    $("body .operator-select-box-right input[type=text]").on('click', function() {
        $(this).select();
    });


    /* date range picker initialisation ----------------------------------------------------------------------- */

    dateRangePicker.init();

    var _page = dateRangePicker.pageLocation;
    var $dr = dateRangePicker.$dr;

    var fUrl = dateRangePicker.urlFormat;
    var fData = dateRangePicker.dataFormat;

    // page specific on change and init events

    switch(dateRangePicker.page) {

        case _page.revenueStreams :

            var revenueUrl = backend_url("insights/revenue-streams");

            dateRangePicker.OnChange(function(label, start, end) {
                loadRevenueStream(moment(start).format(fData), moment(end).format(fData), label);
                dateRangePicker.pushUrl(revenueUrl);
            });

            window.onpopstate = function(event) {
                dateRangePicker.popState(event.state);
                loadRevenueStream(moment(event.state.start).format(fData), moment(event.state.end).format(fData), event.state.label);
            };

            window.setTimeout(function() {

                var start = $dr.attr('data-startdatestr');
                var end = $dr.attr('data-enddatestr');
                var label = $dr.attr("data-period-type");

                loadRevenueStream(start, end, label);
                dateRangePicker.replaceUrl(revenueUrl);

            }, 10);

            break;

        case _page.operatorFunnel :

            var funnelUrl = backend_url("insights/operator-funnel");

            dateRangePicker.OnChange(function(label, start, end) {
                loadOperatorFunnel(moment(start).format(fData), moment(end).format(fData), label);
                dateRangePicker.pushUrl(funnelUrl);
            });

            window.onpopstate = function(event) {
                dateRangePicker.popState(event.state);
                loadOperatorFunnel(moment(event.state.start).format(fData), moment(event.state.end).format(fData), event.state.label);
            };

            window.setTimeout(function() {

                var start = $dr.attr('data-startdatestr');
                var end = $dr.attr('data-enddatestr');
                var label = $dr.attr("data-period-type");

                loadOperatorFunnel(start, end, label);
                dateRangePicker.replaceUrl(funnelUrl);

            }, 10);

            break;

        case _page.payments :

            window.onpopstate = function(event) {
                dateRangePicker.popState(event.state);
                refreshTourStats(dateRangePicker.options({ state : true, mask: true }));
            };

            window.setTimeout(function() {

                refreshTourStats(dateRangePicker.options({ push_url : false, mask : true }));

            }, 10);

            break;


        case _page.dashboard :

            window.onpopstate = function(event) {
                dateRangePicker.popState(event.state);
                refreshTourStats(dateRangePicker.options({ state : true, mask: true }));
            };

            window.setTimeout(function() {

                refreshTourStats(dateRangePicker.options({ push_url : false, mask : true }));

            }, 10);

            break;
    }

    /* todo:OBSOLETE

    if ($(".tour-table").length || [_page.tours, _page.operators, _page.requests].indexOf(this.page) >= 0) {
        dateRangePicker.OnChange(function(label, start, end) {
            // Stats load when date range is changed
            if (window.refreshTourStats) {
                var options = dateRangePicker.popState();
                options.do_push = true;
                refreshTourStats(options);
            }
        });
    }
    */

    /* -------------------------------------------------------------------------------------------------------- */

    $(".account-form input").on('focus', function() {
        if ($(this).attr('placeholder')) {
            $(this).attr('data-placeholder', $(this).attr('placeholder'));
            $(this).removeAttr('placeholder');
        }
    });
    $(".account-form input").on('blur', function() {
        if ($(this).attr('data-placeholder')) {
            $(this).attr('placeholder', $(this).attr('data-placeholder'));
            $(this).removeAttr('data-placeholder');
        }
    });

    $("body").on('click', '.toggle-switch-inner span', function() {

        $(this).parent().toggleClass('toggle-switch-inner-on');
        
        if ($(this).parent().attr('data-action')) {
            if ($(this).parent().attr('data-action') == 'bookedElsewhere') {
                if ($(this).parent().hasClass('toggle-switch-inner-on')) {
                    $(".bookedelsewhere").show();
                } else {
                    $(".bookedelsewhere").hide();
                }
            } else if ($(this).parent().attr('data-action') == 'numberOfRequests') {
                if ($(this).parent().hasClass('toggle-switch-inner-on')) {
                    $(".numberofrequests").show();
                } else {
                    $(".numberofrequests").hide();
                }
            }
        }
    });

    $("body").on('click', '[data-role=switch]', function() {
        var switchId = $(this).attr('data-switch');
        if($(this).attr('data-state') != 'active') {
            // if part of group, deactivate other in group
            if($(this)[0].hasAttribute('data-switchgroup')) {
                $('[data-switchgroup='+$(this).attr('data-switchgroup')+'][data-switch!='+switchId+']').attr('data-state', 'inactive');
            }
            $('[data-switch='+switchId+']').attr('data-state', 'active');
            if($('[data-role=switched][data-switch='+switchId+']').attr('data-switchanimate') == 'height') {
                $('[data-role=switched][data-switch!='+switchId+']').css('height', '');
                $('[data-role=switched][data-switch='+switchId+']').css('height', ($('[data-role=switched][data-switch='+switchId+']')[0].scrollHeight+20)+'px');
            }
        } else {
            $('[data-switch='+switchId+']').attr('data-state', 'inactive');
            $('[data-role=switched][data-switch='+switchId+']').css('height', '');
        }
    });

    $("body").on("click", "#paymentperenquiry .toggle-switch-inner span", function() {
        $(this).parent().parent().next().toggleClass('active');
        if($("#payment-per-country").is(":visible")) {

            $("#payment-per-country").slideUp(800);
        } else {
            $("#payment-per-country").slideDown(800);
        }
        return false;
    });

    $("body").on("click", "#paymentperenquiry .edit-perc", function() {
        var parent = $(this).parent();
        var perc = $(this).attr("data-perc");
        var id = $(this).data("val");
        var type = $(this).data("val").substring(0,1);
        var lower_av = ($(this).data("lower-available") == 1);
        var safari_id;
        $("table.countries-table div.set-payment-per-country").remove();
        parent.append("<div class=\"set-payment-per-country\">"+$("#payment-per-country > .set-payment-per-country").clone().html()+"</div>");

        if (parent.find("#country-percentage").length && parent.find("#country-percentage option[value='2.5']").length) {
            if (lower_av) {
                if (!parent.find("#country-percentage option[value='0.5']").length) {
                    parent.find("#country-percentage").append("<option value='0.5'>0.5 %</option>");
                } else {
                    parent.find("#country-percentage option[value='2.5']").remove();
                }
            }
        }

        if(type=="c")
            parent.find(".country-name").html(parent.prev().find(".country-name").html());
        else {
            parent.find(".set-payment-per-country").find(".country-name").html(parent.prev().find(".country-name").html());
            parent.find(".set-payment-per-country").find(".about").find(".country-name").html("these countries");
        }

        parent.find(".country-flag").html(parent.prev().find(".country-flag").html());
        parent.find(".perc").html($("#quotepercentage").val());
        parent.find("input.default").data("val", id);
        if(parent.parent().hasClass("changed")) {
            parent.find("#country-percentage").val(perc);
            parent.find("input[type=radio].change").prop("checked", true);
        }

        if(typeof safari_id === "undefined")
            safari_id = $("#payment_safariid").val();

        $.ajax({
            type: "POST",
            url: "/requests/countrystats",
            data: {
                safari_id: safari_id,
                operator_id: $("#operator_id").val(),
                type: $(this).data("val").substring(0,1),
                id: $(this).data("val").substring(2)
            }
        }).done(function(html) {
            parent.find(".country-stats").html(html).addClass("done");
            $("table.countries-table div.set-payment-per-country").find(".help").removeAttr("initialized");
            initHelpMarks();
            $('body').click(function(e){
                if ((!$(e.target).parents('.set-payment-per-country').length && !$(e.target).hasClass('set-payment-per-country')) || ($(e.target).parents('.set-payment-per-country').length && $(e.target).hasClass("close-modal"))) {
                    if($(".set-payment-per-country").is(":visible")) {
                        $(".set-payment-per-country").hide();
                    }
                }
            });
        });
        return false;
    });

    $("body").on("click", "#paymentperenquiry .close-modal", function() {
        $(this).parent().remove();
        return false;
    });

    $("body").on("click", "#paymentperenquiry #country-percentage", function() {
        $("input[name=payment-per-country].change").trigger("click");
    });

    $("body").on("click", "#paymentperenquiry input[type=radio].checklabel.default", function() {
        if($(this).parents(".percentage").find(".edit-perc:not(.edit-perc-icon)").attr("data-perc") == "default")
            return true;
        var perc_el = $(this).parents(".percentage");
        $(".set-payment-per-country .loading").show();
        var ts = new Date().getTime();
        var end_ts = ts + 1000;

        setTimeout(function () {
            $("table.countries-table .set-payment-per-country").fadeOut(function() {
                $(this).remove();
                setPercToDefault(perc_el);
            });
        }, 1000);

        return false;
    });

    $("body").on("change", "#quotepercentage", function( event, type ) {

        var isDisabled = $(this).find(":selected").hasClass('disabled');

        showHideError(isDisabled, $(this).closest("p"), 'The maximum payment percentage you can select is 2.0%. If you would like to offer a higher payment percentage, please schedule a consultation with our support team at <a href="mailto:support@AfricaGoTrips.com" target="_blank">support@AfricaGoTrips.com</a>.', 'paymentperenquiry');

        if ($("#overall-errors").length)
            showHideError(isDisabled,$("#test-msg-payment"), 'You need to change the payment percentage to 2.0% or lower.', 'percentage01');

        if(!isDisabled) {
            $("div.buttons input[type=submit].save-percentage").removeClass("inactive");
            showHideError(false, $("div.buttons").find("div.placeholder-for-error"), 'The selected payment percentage cannot be saved. Please select a payment percentage of 2.0% or lower.', 'paymentperenquiry');
            if($(this).val() > 0) {
                $(".country-specific-payment").removeClass("hide");
            } else {
                $(".country-specific-payment").addClass("hide");
            }

            if (typeof type === 'undefined') { clearUndo(); }

            hideError($("#overall-errors"), 'You need to change the payment percentage to 2.0% or lower.', 'percentage01');

        } else {
            $("div.buttons input[type=submit].save-percentage").addClass("inactive");
        }
    });

    $("body").on("change", "#paymentperenquiry #country-percentage", function() {

        if($(this).find(":selected").hasClass('disabled')) {
            if(!$(this).closest("ul").find("li.error").length) {
                $(this).closest("li").after("<li class='error'>error</li>");
            }
            return false;
        } else {
            var perc_el = $(this).parents(".percentage");
            var perc = $(this).val();

            if(perc == "") {
                return false;
            }
            
            $(".set-payment-per-country .loading").show();
            var ts = new Date().getTime();
            var end_ts = ts + 1000;

            setPerc(perc_el, perc);
            setTimeout(function () {
                $("table.countries-table .set-payment-per-country").fadeOut(function() {
                    $(this).remove();
                });
            }, 500)
        }

        return false;
    });

    $("body").on('click', '#copy-payment-start', function() {
        $("#copy-from").val('');
        $("#copy-from").chosenImage({
            disable_search_threshold: 999,
            no_results_text: "Nothing to copy from",
            width: "410px"
        });

        $(this).parents("#copy-payment-settings").find(".copy-function").show();
        $(this).parents("#copy-payment-settings").find(".copy-start").hide();

        $("body").off('click', '.copy-function #cancel-copy');
        $("body").on('click', '.copy-function #cancel-copy', function() {
            $(this).parents("#copy-payment-settings").find(".copy-function").hide();
            $(this).parents("#copy-payment-settings").find(".copy-start").show();
            $(this).removeClass("loading");
            $(".copy-start > span").hide();
            $(".copy-start > a#undo-copy").hide();
            return false;
        });

        $("body").off('click', '.copy-function #apply-copy');
        $("body").on('click', '.copy-function #apply-copy', function() {
            if($("#copy-from").val() == "")
                return false;

            var clicked_button = $(this);

            $("li.default-payment, li.country-specific-payment").addClass("loading");
            $("li.default-payment:before").css("height", $("li.default-payment").height());
            $("li.country-specific-payment:before").css("height", $("li.country-specific-payment").height());

            $("body").off('click', '#undo-copy');
            $("body").on('click', '#undo-copy', function() {

                $("li.default-payment, li.country-specific-payment").addClass("loading");
                $("li.default-payment:before").css("height", $("li.default-payment").height());
                $("li.country-specific-payment:before").css("height", $("li.country-specific-payment").height());
                $(".copy-start > a:not(#copy-payment-start), .copy-start > span").fadeOut(500, function() {
                    $(".copy-start > span, .copy-start > a:not(#copy-payment-start)").hide();
                    //$(".copy-start a").eq(0).show();
                    //$(".copy-start span").eq(0).html("Payment settings restored").show();
                    $(".copy-start span").eq(0).html("Payment settings restored");

                    var undo = JSON.parse($("#undo_payments").val());
                    var all_default = true;
                    for (var key in undo) {
                        var el = "";
                        if (undo.hasOwnProperty(key)) {
                            if (key == "default") {
                                if ($("#quotepercentage").val() != undo[key]) {
                                    $("#quotepercentage").val(undo[key]);
                                    $("#quotepercentage").trigger("change", "trigger");
                                }
                            } else {
                                el = $(".countries-table tr").find("a[data-val=" + key + "]").eq(0).parent();
                                el.find("a").attr("data-perc", undo[key]["perc"]);
                                el.find("a:not(.edit-perc-icon)").attr("data-original-perc", undo[key]["original_perc"]);
                                if (undo[key]["perc"] == "default") {
                                    el.parent().removeClass("changed");
                                    el.find("a:not(.edit-perc-icon)").html(parseFloat(undo["default"]).toFixed(1) + "%");
                                } else {
                                    el.parent().addClass("changed");
                                    el.find("a:not(.edit-perc-icon)").html(parseFloat(undo[key]["perc"]).toFixed(1) + "%");
                                    all_default = false;
                                }
                            }
                        }
                    }
                    if (all_default) {
                        if ($("#paymentperenquiry .toggle-switch .toggle-switch-inner").hasClass("toggle-switch-inner-on")) {
                            $("#paymentperenquiry .toggle-switch .toggle-switch-inner span").eq(0).trigger("click");
                        }
                    } else {
                        if (!$("#paymentperenquiry .toggle-switch .toggle-switch-inner").hasClass("toggle-switch-inner-on")) {
                            $("#paymentperenquiry .toggle-switch .toggle-switch-inner span").eq(0).trigger("click");
                        }
                    }

                    $(".copy-start > span").eq(0).fadeIn(250);
                    $("li.default-payment, li.country-specific-payment").removeClass("loading");
                });
                return false;
            });

            // Save current settings in hidden field.
            var undo = {};
            undo["default"] = $("#quotepercentage").val();
            $.each($(".countries-table tr").find("a.edit-perc:not(.edit-perc-icon)"), function() {
                undo[$(this).attr("data-val")] = {};
                undo[$(this).attr("data-val")]["perc"] = $(this).attr("data-perc");
                undo[$(this).attr("data-val")]["original_perc"] = $(this).attr("data-original-perc");
            });
            $(this).addClass("loading");
            $("#cancel-copy").hide();
            $("#undo_payments").val(JSON.stringify(undo));

            $.ajax({
                type: "POST",
                url: "/billing/copypercentages",
                data: {
                    id: $("#copy-from").val(),
                    operator_id: $("#operator_id").val()
                }
            }).done(function(data) {

                $(".copy-function").delay(200).fadeOut(function() {
                    var i;
                    var percentages = data;
                    var default_perc = parseFloat(percentages["default"]["quote_perc"]);

                    $("#quotepercentage").val(default_perc);
                    $("#quotepercentage").trigger("change", "trigger");

                    var array_length = Object.keys(percentages).length;

                    $(".countries-table tr").removeClass("changed");
                    $(".countries-table tr").find("a.edit-perc").attr("data-perc", "default");
                    $(".countries-table tr").find("a.edit-perc:not(.edit-perc-icon)").html(parseFloat(default_perc).toFixed(1) + "%");

                    if(array_length > 1) {
                        if(!$("#paymentperenquiry .toggle-switch .toggle-switch-inner").hasClass("toggle-switch-inner-on")) {
                            $("#paymentperenquiry .toggle-switch .toggle-switch-inner span").eq(0).trigger("click");
                        }
                    } else {
                        if($("#paymentperenquiry .toggle-switch .toggle-switch-inner").hasClass("toggle-switch-inner-on")) {
                            $("#paymentperenquiry .toggle-switch .toggle-switch-inner span").eq(0).trigger("click");
                        }
                    }

                    for (i = 0; i < array_length; i++) {

                        if(typeof percentages[i] === "undefined")
                            continue;

                        var id = 0;
                        var perc = "";
                        var type = "c";
                        if(percentages[i]["country_all_id"] == 0) { // region perc
                            id = percentages[i]["region_id"];
                            type = "r";
                        } else { // country perc
                            id = percentages[i]["country_all_id"];
                        }
                        id = type + "-" + id;
                        perc = Math.round( percentages[i]["quote_perc"] * 10) / 10

                        $(".countries-table tr").find("a[data-val=" + id + "]:not(.edit-perc-icon)").html(parseFloat(perc).toFixed(1) + "%");
                        $(".countries-table tr").find("a[data-val=" + id + "]").attr("data-perc", perc);
                        $(".countries-table tr").find("a[data-val=" + id + "]:not(.edit-perc-icon)").parent().parent().addClass("changed");
                    };


                    $("#cancel-copy").show();
                    $(".copy-start, .copy-start span, .copy-start a").fadeIn(function() {
                        clicked_button.removeClass("loading");
                    });
                    $("li.default-payment, li.country-specific-payment").removeClass("loading");
                });

            });

            return false;
        });

        return false;
    });

    $("body").on('click', '.history-log li', function() {
        $(this).toggleClass('opened');
    });

    if ($('.history-log-container').length) {
        initHistoryLog();
    }

    if ($('a#account-link').length) {
        $('a#account-link').on('click', function() {
            if (!$('div#account-links').length) {
                $(this).css("background-position", "right -10px");
                $(this).after("<div id=\"account-links\" class=\"account-links\"><ul>" +
                    ((!$(".signasoperator").length || $(".signasoperator").hasClass('out')) ? "<li><a href=\""+$sb.backend_url("company/edit")+"\">Company Profile</a></li>" : "") +
                    "<li><a href=\""+$sb.backend_url("account/out-of-office")+"\">Out of Office</a></li>" +
                    ($(this).hasClass("has-mpr") ? "<li><a href=\""+$sb.backend_url("progress-reports")+"\">Progress Reports</a></li>" : "") +
                    ((!$(".signasoperator").length || $(".signasoperator").hasClass('in')) ? "<li><a href=\""+$sb.backend_url("account/change-password")+"\">Change Password</a></li>" : "") +
                    (($(this).hasClass("has-2fa")) ? "<li><a href=\""+$sb.backend_url("account/2fa")+"\">2 Factor Authentication</a></li>" : "") +
                    ("<li><a href=\""+$sb.backend_url("account/safarioffice")+"\">SafariOffice</a></li>") +
                    (($(this).hasClass("has-api")) ? "<li><a href=\""+$sb.backend_url("account/api")+"\">API</a></li>" : "") +
                    (($(this).hasClass("has-compchecker")) ? "<li><a href=\""+$sb.backend_url("tool/competitionchecker")+"\">Competition Checker</a></li>" : "") +
                    "<li><a class=\"signoffoperator\" href=\"/signout\">Sign Out</a></li>" +
                    "</ul></div>");
                $('body').click(function(e){
                    if ((!$(e.target).parents('#account-links').length && !$(e.target).hasClass('account-links'))) {
                        if($('div#account-links').is(":visible")) {
                            $('div#account-links').remove();
                            $('a#account-link').css("background-position", "right 7px");
                        }
                    }
                });
                $("a.signoffoperator").off();
            } else {
                $("div#account-links").remove();
                $(this).css("background-position", "right 7px");
            }
            return false;
        });
    }

    var $endOOO = $(".end-ooo");
    if($endOOO.length) {
        $endOOO.off();
        $endOOO.on('click', function () {
            var el = $(this).parents("#outofoffice-notice");
            var reload = true;
            if(!$("input[name=end_ooo]").length) {
                el.html("&nbsp;");
                reload = false;
            }
            $.post("/account/out-of-office",
                { 'delete': 1 }
            ).done(function () {
                if(reload) {
                    location.reload();
                } else {
                    if ($("#start-data-point").length) {
                        refreshTourStats();
                    }
                    el.html("<b>Your 'Out of Office' has been turned off</b>").delay(6000).slideUp(500);
                }
            });
        });
    }

    $(document).on("click", "#submit-decline", function() {
        if($('input[name=name]').val() == '') {
            alert("Please enter your name.");
        } else if(!$('input[name=email]').val().match(/^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/)) {
            alert("Please enter a valid email address.");
        } else {
            $(this).addClass("busy");
            $(this).closest("form").submit();
        }
    });

    $(document).on("click", "[data-role=preview-ooo]", function() {
        
    });

    if($("input[type=hidden][name=load_ajax_url]").length && $("input[type=hidden][name=load_ajax_url]").val() != "") {
        var load_ajax_url = $("input[type=hidden][name=load_ajax_url]").val();

        $.ajax(load_ajax_url);

    }

    $(".photochecker-tool select").on('change', function(e) {
        var loc ='/tool/photochecker';

        if ($(".photochecker-tool select[name='id']").val()) {
            loc += '?id=' + $(".photochecker-tool select[name='id']").val();
        }

        if ($(".photochecker-tool select[name='type']").val()) {
            loc += ((loc.indexOf('?') > 0) ? '&' : '?') + 'type=' + $(".photochecker-tool select[name='type']").val();
        }

        location.href = loc;
    });

    $(".photochecker-tool--body a").on('click', function(e) {

        if (!$(this).hasClass('no')) {
            e.preventDefault();
            $(this).addClass('no');
            var id = $(this).attr('data-id');
            var type = $(".photochecker-tool select[name='type']").val();

            $.post( "/tool/photochecker", { "photoid": id, "type": type }, function(  ) {
            });
        } else {
            e.preventDefault();
            $(this).removeClass('no');
            var id = $(this).attr('data-id');
            var type = $(".photochecker-tool select[name='type']").val();

            $.post( "/tool/photochecker", { "photoid": id, "type": type, 'on': 1 }, function(  ) {
            });
        }

    });

    if ($(".admin-operatorlifetime").length) {
        loadOperatorLifetime();
    }    
    
    refreshCSRFToken();

});

function updateDefaultPercentages(perc) {
    $.each($(".countries-table tr"), function() {
        if($(this).find(".edit-perc:not(.edit-perc-icon)").attr("data-perc") == "default") {
            $(this).find(".edit-perc:not(.edit-perc-icon)").html(parseFloat(perc).toFixed(1) + "%");
        }
    });
}

function fillHiddenPaymentFields() {
    // Gather country specific payment percentages and process them in hidden inputs
    var to_default = new Array();
    var changes = false;
    var multi = $("table.applied-to").length;
    if($("#paymentperenquiry .toggle-switch .toggle-switch-inner").hasClass("toggle-switch-inner-on")) {
        var to_perc = new Array();
        $.each($(".countries-table tr .edit-perc:not(.edit-perc-icon)"), function() {
            var from = $(this).attr("data-original-perc");
            var to = $(this).attr("data-perc");
            var id = $(this).attr("data-val");
            //if(multi || from != to) {
                changes = true;
                if(to == "default") {
                    to_default.push(id);
                } else {
                    to_perc.push(id+"-"+to);
                }
            //}
        });
        $("input[type=hidden][name=country_payment_perc]").val(JSON.stringify(to_perc));
    } else {
        $.each($(".countries-table tr .edit-perc:not(.edit-perc-icon)"), function() {
            var from = $(this).attr("data-original-perc");
            var to = "default";
            var id = $(this).attr("data-val");
            //if(multi || from != to) {
                changes = true;
                to_default.push(id);
            //}
        });
        $("input[type=hidden][name=country_payment_perc]").val("");
    }
    $("input[type=hidden][name=country_payment_default]").val(JSON.stringify(to_default));
    return changes;
}

function setPerc(perc_el, perc) {
    perc_el.find(".edit-perc").attr("data-perc",perc);
    perc_el.find(".edit-perc:not(.edit-perc-icon)").html(parseFloat(perc).toFixed(1) + "%");
    perc_el.parent().addClass("changed");
    clearUndo();
    if($("table.applied-to").length)
        $billing.updatePayPerc();
}

function setPercToDefault(perc_el) {
    perc_el.find(".edit-perc").attr("data-perc","default");
    perc_el.find(".edit-perc:not(.edit-perc-icon)").html(parseFloat($("#quotepercentage").val()).toFixed(1) + "%");
    perc_el.parent().removeClass("changed");
    clearUndo();
    if($("table.applied-to").length)
        $billing.updatePayPerc();
}

function clearUndo() {
    if($(".copy-start").is(":visible")) {
        $(".copy-start span").eq(0).fadeOut("normal", function() {
            $(".copy-start span").eq(0).html("Payment settings copied");
        });
        $(".copy-start span").eq(1).fadeOut(250);
        $(".copy-start a").eq(1).fadeOut(250);
    }
}

var history_log_init = false;

function vertHeight() {
    var new_height = parseInt($(window).height()) - 135;
    if (new_height < 422) {
        new_height = 422;
    }
    return new_height;
}

function closeEmailBox() {

    var email_box = $(".email-box");
    var email_box_holder = $(".email-box-holder");

    var new_bottom = -444;

    var form_height = parseInt(email_box.find('.form').css('height'));
    if (form_height > 422) {
        new_bottom -= (form_height - 422);
    }
    email_box_holder.stop().animate({
        bottom: new_bottom + 'px'
    }, 300, function(){
        email_box.addClass("close").removeClass("open");
        historyOpenButton();
    });

}

function historyOpenButton() {

    $hist = $(".history-box-holder");
    $email = $(".email-box-holder");

    var email_box = $(".email-box");
    var history_box = $(".history-box");

    if (email_box.hasClass('open') && history_box.hasClass('open')) {

        if ($hist.offset().left == $email.offset().left && $(".history-box-holder .open-button").is(":visible")) {
            $(".history-box-holder .open-button").fadeOut();
        }
        else
        if ($hist.offset().left != $email.offset().left && !$(".history-box-holder .open-button").is(":visible")) {
            $(".history-box-holder .open-button").fadeIn();
        }
    }
    else {
        if (!$(".history-box-holder .open-button").is(":visible")) {
            $(".history-box-holder .open-button").fadeIn();
        }
    }
}

function emailBoxDimensions() {

    $hist = $(".history-box-holder");
    $prob = $(".problem-box-holder");

    if ($hist.length && $prob.length) {

        $email = $(".email-box-holder");

        var ho = $hist.offset();
        var po = $prob.offset();

        var availWidth = (po.left) - (ho.left + $hist.width())-37;
        var widthNeeded = $email.width();

        $email.css("left", ho.left + ((availWidth >= widthNeeded)?($hist.width()-9):0));

        var email_box = $(".email-box");
        if (email_box.hasClass('open')) {
            email_box.find('.form').css('height', vertHeight());
            historyOpenButton();
        }

    }

}

function historyBoxDimensions() {

    var history_box = $(".history-box");
    if (history_box.hasClass('open')) {

        var new_height = vertHeight();
        history_box.find('.form').css('height', new_height);

        history_box.find('.history_log_container').css('height', 237 + (new_height - 422));
        history_box.find('.history-log.scrolled').css('height', 200 + (new_height - 422));
    }
}

function problemBoxDimensions() {
    var problem_box = $(".problem-box");
    if (problem_box.hasClass('open')) {

        var new_height = vertHeight();
        problem_box.find('.form').css('height', new_height-20);

        var textarea_height_diff = new_height - 445;
        var before_message_elements = $( "#mail-operator-txt" ).prevAll();
        var before_message_height = 0;
        before_message_elements.each(function() {
            if($(this).is(':visible')) {
                before_message_height += $(this).outerHeight();
            }
        });
        $("#mail-operator-txt").css('height', 364 - before_message_height + textarea_height_diff);
        $("#mail-operator-txt_tbl").css('height', 371 - before_message_height + textarea_height_diff);
        $("#mail-operator-txt_lbl").css('height', 364 - before_message_height + textarea_height_diff);
        $("#mail-operator-txt_ifr").css('height', 341 - before_message_height + textarea_height_diff);
    }
}

function openEmailBox(el) {

    var email_box = $(".email-box");
    var email_box_holder = $(".email-box-holder");

    $funcs.jsonCall("/emails/getdata", { eid : el.data("eid") }, function(data){

        $("[data-role='email-to']").html(data.to);
        $("[data-role='email-from']").html(data.sb_name+" &lt;"+data.sb_email+"&gt;");
        $("[data-role='email-sent']").text(data.date);
        $("[data-role='email-subject']").text(data.subject);
        $("div.email-box-body").html(data.body);

    });

    if (!email_box.hasClass('open')) {

        email_box.find('.form').css('height', vertHeight());

        email_box_holder.stop().animate({
            bottom: 0
        }, 300, function(){
            email_box.addClass("open").removeClass("close");
            historyOpenButton();
        });
    }

    email_box.off("click","a.email-close");
    email_box.off("click",".form");
    email_box.on("click","a.email-close", function(e){
        e.preventDefault();
        closeEmailBox();
        return false;
    });

    email_box.on("click",".form", function(e){
        e.preventDefault();
        return false;
    });

}


function initHistoryLog() {

    if (history_log_init) {
        loadHistoryLog();
        return true;
    }

    delayed500(loadHistoryLog);

    $(".history-box-holder").on('click', '.open-button', function(e) {
        e.preventDefault();
        var history_box = $(".history-box");
        var history_box_holder = $(".history-box-holder");
        if (!history_box.hasClass('open')) {

            var new_height = vertHeight();
            history_box.find('.form').css('height', new_height);
            history_box.find('.history_log_container').css('height', 237 + (new_height - 422));
            history_box.find('.history-log.scrolled').css('height', 200 + (new_height - 422));

            history_box_holder.stop().animate({
                bottom: 0
            }, 300, function(){
                history_box.addClass("open").removeClass("close");
                history_box.find('textarea').focus();
            });


        } else {

            var new_bottom = -444;

            var form_height = parseInt(history_box.find('.form').css('height'));
            if (form_height > 422) {
                new_bottom -= (form_height - 422);
            }

            history_box_holder.stop().animate({
                bottom: new_bottom + 'px'
            }, 300, function(){
                history_box.addClass("close").removeClass("open");
            });

            if ($(".email-box").hasClass('open'))
                closeEmailBox();

        }
    });

    $(".log-history-box").on('click', 'li.show-all a', function(e) {
        e.preventDefault();
        $(".history-log").toggleClass('showall');
    });
    $(".history-log-container").on('mouseenter', '.history-log .toggle-switch', function() {
        if ($(this).closest('.solv').find('.solved-lbl').html() != '') {
            $(this).closest('.solv').find('.solved-lbl').stop().fadeIn();
        }
    });
    $(".history-log-container").on('mouseleave', '.history-log .toggle-switch', function() {
        $(this).closest('.solv').find('.solved-lbl').stop().fadeOut();
    });
    $(".history-log-container").on('mouseenter', '.history-log .date', function() {
        $(this).find('div').stop().fadeIn();
    });
    $(".history-log-container").on('mouseleave', '.history-log .date', function() {
        $(this).find('div').stop().fadeOut();
    });
    $(".history-log-container").on('click', '.history-log .toggle-switch-inner span', function(e) {
        e.preventDefault();
        var $elem = $(this);
        $elem.closest('li').toggleClass('solved');

        var data =  {
            "operator_log_id": $elem.closest('li').data('id'),
            "solved": $elem.closest('li').hasClass('solved')
        };

        // check if log of operator with id
        if ($elem.closest('.history-log-container').data('opid')) {
            data.operator_id = $elem.closest('.history-log-container').data('opid');
        }

        // check if log of operator signup
        if ($elem.closest('.history-log-container').data('signupid')) {
            data.signup_id = $elem.closest('.history-log-container').data('signupid');
        }

        $.post("/admin/historylogsolved", data, function() {
            if ($elem.closest('li').hasClass('solved')) {
                $elem.closest('li').find(".solv > .solved-lbl").html('Solved: ' + $elem.closest('ul').data('current'));
            } else {
                $elem.closest('li').find(".solv > .solved-lbl").html('').hide();
            }

            if ($(".history-log-container.only-unsolved").length) {
                loadHistoryLog();
            }
        });
    });
    $(".history-box").on('click', '.prio-all .toggle-switch-inner span', function(e) {
        e.preventDefault();
        if (!$('.history-box').find(".prio-all").find(".toggle-switch-inner").hasClass("toggle-switch-inner-on")) {
            $('.history-box').find(".history-log.scrolled li.lowprio").hide();
        } else {
            $('.history-box').find(".history-log.scrolled li.lowprio").show();
        }
    });
    $(".history-box").on('click', 'a.email-open', function(e) {
        e.preventDefault();
        openEmailBox($(this));
        return false;
    });
    $(".history-log-container").on('click', '.history-log .actions a', function(e) {
        e.preventDefault();
        if ($(this).hasClass('edit')) {
            editHistoryLog($(this).closest('li').data('id'));
        } else if ($(this).hasClass('delete')) {
            var elem = $(this);
            confirm('Are you sure you want to remove this History Note?', function(result) {
                if (result) {
                    deleteHistoryLog($(elem).closest('li').data('id'));
                }
            });
        }
    });
    $(".new-history-log .sb-button").bind('click', function(e) {
        e.preventDefault();
        if ($(this).hasClass('btn-add')) {
            saveHistoryLog();
        } else {
            $(".new-history-log textarea").val('');
            $(this).closest('.history-box-holder').find('.open-button').trigger('click');
        }
    });
    $(".new-history-log #call").bind("change", function(e) {

        var $comment = $(".new-history-log textarea");
        var txt =  $(this).parent().text().trim();

        if ($(this).is(":checked") && $comment.val() === "") {
            $comment.val(txt);
        }
        else
        if (!$(this).is(":checked") && $comment.val() === txt) {
            $comment.val("");
        }
    });


    history_log_init = true;
}

function saveHistoryLog() {
    if ($(".new-history-log textarea").val() == '') {
        alert('Please enter a comment first');
    } else {
        if($(".new-history-log").data('accoid') && $(".new-history-log").data('accoid') > 0) {

            var data = {
                "accommodation_id": $(".new-history-log").data('accoid'),
                "type": "Note",
                "call": ($("#call").is(":checked") ? 1 : 0),
                "comment": $(".new-history-log textarea").val()
            };

            $.post("/admin/historylogset", data, function(data) {
                loadHistoryLog();
                $(".new-history-log textarea").val('');

                if ($("#last-call").length)
                    $("#last-call").text(data.last_call);
            });
        } else {

            var data = {
                "type": ($("#solvable").is(":checked") ? 'SolvableNote' : 'Note'),
                "call": ($("#call").is(":checked") ? 1 : 0),
                "comment": $(".new-history-log textarea").val()
            };

            // check if log of operator with id
            if ($(".new-history-log").data('opid')) {
                data.operator_id = $(".new-history-log").data('opid');
            }

            // check if log of operator signup
            if ($(".new-history-log").data('signupid')) {
                data.signup_id = $(".new-history-log").data('signupid');
            }

            // check if we have a safari_id
            if ($(".new-history-log").data('safid')) {
                data.safari_id = $(".new-history-log").data('safid');
            }

            $.post("/admin/historylogset", data, function(data) {
                loadHistoryLog();
                $(".new-history-log textarea").val('');

                if ($("#last-call").length)
                    $("#last-call").text(data.last_call);

            });
        }
    }
}

function deleteHistoryLog(id) {
    if($(".history-log-container.accommodation").length) {
        $.post("/admin/historylogdelete", { "accommodation_log_id": id }, function(data) {
            loadHistoryLog();
            if ($("#last-call").length)
                $("#last-call").text(data.last_call);
        });
    } else {
        $.post("/admin/historylogdelete", { "operator_log_id": id }, function(data) {
            loadHistoryLog();
            if ($("#last-call").length)
                $("#last-call").text(data.last_call);
        });
    }
}

function editHistoryLog(id) {
    addSBLoadLayer(undefined, true);

    var url = '/admin/logedit-modal?id=' + id;
    if($(".new-history-log").data('accoid') && $(".new-history-log").data('accoid') > 0) {
        url += '&type=acco';
    }

    $("#reveal_modal_center").foundation('reveal', 'open', {
        url: url,
        success: function(data) {
            $("#reveal_modal_center").addClass("logedit-reveal-modal");
        },
        error: function() { },
        animation: 'fade',
        close_on_background_click: false
    });

    $(".close-reveal-modal").remove();
    $(".reveal-modal-bg").hide();

    $(document).on('closed.fndtn.reveal', '#reveal_modal_center', function () {
        $("#reveal_modal_center").removeClass("logedit-reveal-modal");
        handleCloseRevealModal();
    });

    $(document).on('opened.fndtn.reveal', "#reveal_modal_center", function () {
        handleOpenRevealModal();

        $(".logedit-reveal-modal").find('.apply').unbind('click').bind('click', function(e) {
            e.preventDefault();
            comment = $("#reveal_modal_center").find('textarea').val();
            if($(".history-log-container.accommodation").length) {
                $.post("/admin/historylogedit", { "accommodation_log_id": id, "comment": comment, "call" : ($("#call").is(":checked")?1:0), "type": ($("#solvable").is(":checked") ? 'SolvableNote' : 'Note') }, function(data) {
                    if (data.status == "error" && onError!=null)
                        self.createAlert(el,"error", 'Something went wrong, try again');
                    else {
                        loadHistoryLog();
                        $("#reveal_modal_center").foundation('reveal', 'close');
                        if ($("#last-call").length)
                            $("#last-call").text(data.last_call);
                    }
                });
            } else {
                $.post("/admin/historylogedit", { "operator_log_id": id, "comment": comment, "call" : ($("#call").is(":checked")?1:0), "type": ($("#solvable").is(":checked") ? 'SolvableNote' : 'Note') }, function(data) {
                    if (data.status == "error" && onError!=null)
                        self.createAlert(el,"error", 'Something went wrong, try again');
                    else {
                        loadHistoryLog();
                        $("#reveal_modal_center").foundation('reveal', 'close');
                        if ($("#last-call").length)
                            $("#last-call").text(data.last_call);
                    }
                });
            }
        });
        $(".logedit-reveal-modal").find('.cancel').unbind('click').bind('click', function(e) {
            e.preventDefault();
            $("#reveal_modal_center").foundation('reveal', 'close');
        });
    });

}

function loadHistoryLog() {

    $historyContainer = $(".history-log-container");
    
    if(!$historyContainer.length) {
        return;
    }

    if($historyContainer.data("signupid")) {
        var url = "/admin/historylog?signup_id=" + $historyContainer.data('signupid');
    }
    else
    if($historyContainer.hasClass("accommodation")) {
        var url = "/admin/historylog?accommodation_id=" + $('.history-log-container.accommodation').data('acid');
    } else {
        var url = "/admin/historylog?operator_id=" + $historyContainer.data('opid');
        if ($historyContainer.data('safid')) {
            url += "&safari_id=" + $('.history-log-container').data('safid');
        }
        if ($historyContainer.data('safid')) {
            url += "&safari_id=" + $('.history-log-container').data('safid');
        }
    }

    $historyContainer.each(function() {
        if ($historyContainer.find(".history-log").length) {
            $historyContainer.addClass('reloading');
        };
    });

    $historyContainer.load(url, function() {

        $(".history-log-container").removeClass('reloading');
        if ($(".history-log-container.only-unsolved .history-log.scrolled li.highprio").length > 0) {
            $(".history-log-container.only-unsolved").show();

            if (!$(".history-log-container.no-scrolling .open-history").length) {
                $(".history-log-container.no-scrolling").append('<a href="" class="sb-button grey open-history small">Open History</a>');
            }

            $(".open-history").unbind('click').bind('click', function(e) {
                e.preventDefault();
                if (!$(".history-box").hasClass('open')) {
                    $(".history-box .open-button").trigger('click') ;
                }
            });
        } else {
            $(".history-log-container.only-unsolved").hide();
        }

        $(this).find('li.highprio').last().addClass('last')

        if ($(this).closest('.history-box').length) {
            if ($(this).closest('.history-box').find(".prio-all").find(".toggle-switch-inner").hasClass("toggle-switch-inner-on")) {
                $(this).closest('.history-box').find(".history-log.scrolled li.lowprio").hide();
            } else {
                $(this).closest('.history-box').find(".history-log.scrolled li.lowprio").show();
            }

            if ($(this).closest('.history-box').hasClass('open')) {
                var new_height = vertHeight();
                $(this).closest('.history-box').find('.form').css('height', new_height);
                $(this).closest('.history-box').find('.history_log_container').css('height', 237 + (new_height - 422));
                $(this).closest('.history-box').find('.history-log.scrolled').css('height', 200 + (new_height - 422));
            }
        }
    });
}

function closeConfirmBox() {
    $('.confirm-layer').fadeOut('fast', function() { $('.confirm-layer').remove();$("body").css('overflow', 'auto'); });
}

function loadRevenueStream(start, end, label) {

    var $adminRevenueStreams = $(".admin-revenuestreams");
    var $tableRevenueStreams = $adminRevenueStreams.find("table");
    var $loaderRevenueStreams = $adminRevenueStreams.find(".loader");

    $(".admin-revenuestreams tbody tr:not(.loader)").remove();
    $(".admin-revenuestreams tfoot").remove();
    $loaderRevenueStreams.show();

    var pData = {};
    if (label)
        pData["pt"] = label;
    else {
        pData["sd"] = start;
        pData["ed"] = end;
    }

    $helper.jsonCall(backend_url("insights/revenue-streams"), pData, function(data) {
        $.each(data.rows, function (key, row) {
            var html = '<tr>';
            $.each(row, function(key2, val) {
                html += '<td><div>' + val + '</div></td>';
            });
            html += '</tr>';
            $adminRevenueStreams.find("tbody").append(html);
            //console.log(row);
        });

        var html = '<tfoot><tr>';
        $.each(data.totals, function(key2, val) {
            html += '<th><div class="right">' + val + '</div></th>';
        });
        html += '</tr></tfoot>';
        $tableRevenueStreams.append(html);

        $loaderRevenueStreams.hide();
        $tableRevenueStreams.trigger('update');
        $tableRevenueStreams.tablesorter({
            sortRestart: true,
            sortList: [[2,1]],
            headers: {
                0: { sorter: 'text', sortInitialOrder: 'asc' },
                1: { sorter: 'digit', sortInitialOrder: 'desc' },
                2: { sorter: 'digit', sortInitialOrder: 'desc' },
                3: { sorter: 'digit', sortInitialOrder: 'desc' },
                5: { sorter: 'digit', sortInitialOrder: 'desc' },
                6: { sorter: 'digit', sortInitialOrder: 'desc' },
                7: { sorter: 'digit', sortInitialOrder: 'desc' },
                8: { sorter: 'digit', sortInitialOrder: 'desc' },
                9: { sorter: 'digit', sortInitialOrder: 'desc' },
                10: { sorter: 'digit', sortInitialOrder: 'desc' }
            }
        });
    });
    
}

function loadOperatorFunnel(start, end, label) {

    $operatorFunnel = $(".operator-funnel");
    $operatorFunnel.addClass('loading');
    
    var start_w = 1200;
    var start_n = false;

    var pData = {};
    if (label)
        pData["pt"] = label;
    else {
        pData["sd"] = start;
        pData["ed"] = end;
    }

    $helper.jsonCall(backend_url("insights/operator-funnel"), pData, function(data) {
        
        $.each(data, function(key, val) {

            $step = $(".step[data-key='" + key + "']");

            if (start_n === false) {
                start_n = val.continue;
            }

            var c_perc = 0;
            if (val.in > 0) {
                c_perc = Math.round(val.continue / (val.in) * 100);
            }

            $step.find("i").text(val.in);

            if (val.in > 0) {
                if (val.o1 != null) {
                    var o1_perc = Math.round(val.o1 / (val.in) * 100);
                    var o2_perc = Math.round(val.o2 / (val.in) * 100);
                    var o3_perc = Math.round(val.o3 / (val.in) * 100);
                    var o4_perc = Math.round(val.o4 / (val.in) * 100);

                    $step.find(".out1").html("Missing info, waiting for more info: " +val.o1 + " <small>(" + o1_perc + "%)</small>");
                    $step.find(".out2").html("Missing info, waiting for link: " +val.o2 + " <small>(" + o2_perc + "%)</small>");
                    $step.find(".out3").html("Missing info, rejected after upload: " +val.o3 + " <small>(" + o3_perc + "%)</small>");
                    $step.find(".out4").html("Completed, no account yet: " +val.o4 + " <small>(" + o4_perc + "%)</small>");
                } else {
                    if ($step.find(".out").attr('data-label') && $step.find(".out").attr('data-label') != '') {
                        $step.find(".out").html($step.find(".out").attr('data-label') + ': ' + val.stopped + " <small>(" + (100 - c_perc) + "%)</small>");
                    } else {
                        $step.find(".out").html(val.stopped + " <small>(" + (100 - c_perc) + "%)</small>");
                    }
                }
                $step.find(".cnt").html(val.continue + "<br /><small>(" + c_perc + "%)</small>");
            } else {
                if ($step.find(".out").attr('data-label') && $step.find(".out").attr('data-label') != '') {
                    $step.find(".out").html($step.find(".out").attr('data-label') + ': ' + val.stopped);
                } else {
                    $step.find(".out").html(val.stopped);
                }
                $step.find(".cnt").html(val.continue);
            }

            $step.css('width', (start_w * (val.continue / start_n)) + 'px');
            
            $step.find(".continue").css('width', c_perc + "%");
            $step.find(".stopped").css('width', (100 - c_perc) + "%");
            
        });

        $operatorFunnel.removeClass('loading');
    });
}

function loadOperatorLifetime() {
    
    $(".admin-operatorlifetime tbody tr:not(.loader)").remove();
    $(".admin-operatorlifetime .loader").show();
    $helper.jsonCall("/insights/operator-lifetime", { }, function(data) {
        $.each(data.rows_table1, function (key, row) {
            var html = '<tr>';
            $.each(row, function(key2, val) {
                if (typeof val == 'object') {
                    if (val['amount'] == 0) {
                        html += '<td><div><nobr>0</nobr></div></td>';
                    } else {
                        html += '<td><div><nobr><i title="' + val['alt'] + '" style="display: block;font-style:normal;text-decoration: underline;">' + val['amount'] + '</i></nobr></div></td>';
                    }
                } else {
                    html += '<td><div><nobr>' + val + '</nobr></div></td>';
                }
            });
            html += '</tr>';
            $(".admin-operatorlifetime.table1 tbody").append(html);
        });
        
        $.each(data.rows_table2, function (key, row) {
            var html = '<tr>';
            $.each(row, function(key2, val) {
                if (typeof val == 'object') {
                    html += '<td><div><nobr>' + val['amount'] + '</nobr></div></td>';
                } else {
                    html += '<td><div><nobr>' + val + '</nobr></div></td>';
                }
            });
            html += '</tr>';
            $(".admin-operatorlifetime.table2 tbody").append(html);
        });

        $(".admin-operatorlifetime .loader").hide();
    });
    
}

function tinyMCEChanged(inst) {
    var temp = inst.getBody().innerHTML.replace(/<p>/gi,"").replace(/<\/p>/gi,"<br />").replace(/<ul>/gi,"").replace(/<\/ul>/gi,"").replace(/<li>/gi,"").replace(/<\/li>/gi,"<br />");
    if(inst.settings.id == 'full_desc') {
        $("#full_desc2").html(temp);
    }
    if(inst.settings.id == 'departure_dates_text') {
        $("#departure_dates_text2", parent.document).val(temp);
    }
}

function $m(theVar){
    return document.getElementById(theVar)
}

function remove(theVar){
    var theParent = theVar.parentNode;
    theParent.removeChild(theVar);
}

function addEvent(obj, evType, fn){
    if(obj.addEventListener)
        obj.addEventListener(evType, fn, true)
    if(obj.attachEvent)
        obj.attachEvent("on"+evType, fn)
}

function removeEvent(obj, type, fn){
    if(obj.detachEvent){
        obj.detachEvent('on'+type, fn);
    }else{
        obj.removeEventListener(type, fn, false);
    }
}

function addSBLoadLayer(text, blackbg, width) {
    if (typeof text == 'undefined') {
        text = 'Loading, please wait...';
    }
    if (typeof blackbg == 'undefined') {
        blackbg = false;
    }
    if (typeof width == 'undefined') {
        width = 200;
    }
    // Not everything is loaded (form + both auto complete ajax files
    $("body").append('<div class="load-layer' + (blackbg ? ' blackbg' : '') + '"><div><img src="/images/loader.gif" /> ' + text + '</div></div>');
    $(".load-layer > div").css('width', width);
    $(".load-layer").fadeIn('fast');
}

function removeSBLoadLayer() {
    // Form + both auto complete ajax files are load
    $(".load-layer").stop().fadeOut("normal", function() {
        $(".load-layer").remove();
    });
}

function isWebKit(){
    return RegExp(" AppleWebKit/").test(navigator.userAgent);
}

function ajaxUpload(form,url_action,id_element,html_show_loading,html_error_http,callback){
    $("body").mask('Your photo is being uploaded...');

    $('.loadmask-msg').css("top", "40%");
    $('.loadmask-msg').css("margin", "auto");
    $('.loadmask-msg').css("position", "fixed");

    var detectWebKit = isWebKit();
    form = typeof(form)=="string"?$m(form):form;
    var erro="";
    if(form==null || typeof(form)=="undefined"){
        erro += "The form of 1st parameter does not exists.\n";
    }else if(form.nodeName.toLowerCase()!="form"){
        erro += "The form of 1st parameter its not a form.\n";
    }
    if($m(id_element)==null){
        erro += "The element of 3rd parameter does not exists.\n";
    }
    if(erro.length>0){
        alert("Error in call ajaxUpload:\n" + erro);
        return;
    }
    var iframe = document.createElement("iframe");
    iframe.setAttribute("id","ajax-temp");
    iframe.setAttribute("name","ajax-temp");
    iframe.setAttribute("width","0");
    iframe.setAttribute("height","0");
    iframe.setAttribute("border","0");
    iframe.setAttribute("style","width: 0; height: 0; border: none;");
    form.parentNode.appendChild(iframe);
    window.frames['ajax-temp'].name="ajax-temp";
    var doUpload = function(){
        removeEvent($m('ajax-temp'),"load", doUpload);
        var cross = "javascript: ";
        cross += "window.parent.$m('"+id_element+"').innerHTML = document.body.innerHTML; void(0);";
        $m(id_element).innerHTML = html_error_http;
        //alert(html_error_http);
        $m('ajax-temp').src = cross;
        if(detectWebKit){
            remove($m('ajax-temp'));
        }else{
            setTimeout(function(){remove($m('ajax-temp'))}, 250);
        }

        setTimeout(callback + '();', 100);
        $("body").unmask();
    }
    addEvent($m('ajax-temp'), "load", doUpload);
    form.setAttribute("target", "ajax-temp");
    form.setAttribute("action", url_action);
    form.setAttribute("method", "post");
    form.setAttribute("enctype", "multipart/form-data");
    form.setAttribute("encoding", "multipart/form-data");
    form.submit();
    if(html_show_loading.length > 0){
        $m(id_element).innerHTML = html_show_loading;
    }
}

function setAjaxUpload(pars)
{

    var obj = pars["obj"];
    var allowed_image_size = pars["allowed_image_size"];
    var container = pars["container"];
    var callback = pars["callback"];
    var file_name = pars["file_name"];
    var remove_label = pars["remove_label"];
    if(pars["type"] == "file") {
        var file_path = pars["file_path"];
        ajaxUpload(
            obj.form,
            '/img/ajax-upload/file_name/' + file_name + '/remove_label/' + remove_label + '/allowed_image_size/' + allowed_image_size + '/file_path/' + file_path + '/',
            container,
            'File Uploading Please Wait...',
            'Error in Upload, check settings and path info in source code.',
            callback
        );
    } else {
        var keep_ratio = pars["keep_ratio"];
        if(keep_ratio != 1) keep_ratio = 0;
        var allowed_image_width = pars["allowed_image_width"];
        var allowed_image_height = pars["allowed_image_height"];
        var image_original_path = pars["image_original_path"];
        var image_path = pars["image_path"];

        ajaxUpload(
            obj.form,
            '/img/ajax-upload/file_name/' + file_name + '/remove_label/' + remove_label + '/allowed_image_size/' + allowed_image_size + '/allowed_width/' + allowed_image_width + '/allowed_height/' + allowed_image_height + '/image_original_path/' + image_original_path + '/image_path/' + image_path + '/output/imgtag/keepratio/' + keep_ratio,
            container,
            'File Uploading Please Wait...',
            'Error in Upload, check settings and path info in source code.',
            callback
        );
    }
}

function notice(elm, message, type)
{
    if(type=='error')
    {
        addClass = 'error';
        removeClass = 'success';
        boxtitle = 'Status message : Error';
    }
    else
    {
        addClass = 'success';
        removeClass = 'error';
        boxtitle = 'Status message : Success';
    }
    //$(elm).attr('title', boxtitle).addClass(addClass).removeClass(removeClass).html(message).attr('style','display:none;');
    $('#confirmbox').html('<div id="displaybox"></div>').attr('title', boxtitle).addClass('message');
    $('#displaybox').addClass(addClass).removeClass(removeClass).html(message);
    var dialog = $('#confirmbox').dialog();

    setTimeout(function()
    {
        dialog.dialog('destroy');
    },7000); // 5 seconds
}

function countChars(obj, maxchars)
{
    var char_span_id	=	(obj.id)+'_char_count_span';

    var char_entered	=	(obj.value).length;
    var char_left			=	(maxchars)-(char_entered);
    if((char_left==0) || (char_left<0))
    {
        //alert('you can enter only '+maxchars+' characters');
        obj.value	=	(obj.value).substring(0, maxchars);
        char_left=0;
    }
    document.getElementById(char_span_id).innerHTML = char_left;
}

function forceNumeric(ob) {
    var invalidChars = /[^0-9]/gi
    if(invalidChars.test(ob.value)) {
        ob.value = ob.value.replace(invalidChars,"");
    }
}



/**
 Catch input fields with a data-min or data-max and show limit box at the bottom of that fields
 */
function limitedInputs() {
    if (typeof is_tour_form != 'undefined' && is_tour_form && !tour_form_finished) {
        return;
    }

    if (typeof is_tour_form != 'undefined' && is_tour_form && typeof debugFunctionCall != 'undefined') {
        debugFunctionCall('limitedInputs');
    }
    $.fn.regexMask = function (mask, trim, block_enter) {
        if (!mask) {
            /* throw 'mandatory mask argument missing'; */
        } else {
            try {
                mask.test("");
            } catch(e) {
                /* throw 'mask regex need to support test method'; */
            }
        }
        $(this).unbind('keypress').unbind('dragover').unbind('dragenter').unbind('drop').unbind('paste').keypress(function (event) {
            var thisObj = $(this);
            var charCode = event.keyCode || event.which;
            if (!charCode) {
                return true;
            }

            if (block_enter) {
                if (charCode == 13) {
                    if ($('.tour-form').length) {
                        showError(thisObj.next(), 'We\'re sorry, but that character is not allowed', false, 3000);
                    }
                    return false;
                }
            } else {
                if (charCode == 13 && this.value.charCodeAt(this.selectionStart) == 10 && isNaN(this.value.charCodeAt(this.selectionStart + 1))) {
                    if ($('.tour-form').length) {
                        showError(thisObj.next(), 'We\'re sorry, but that character is not allowed', false, 3000);
                    }
                    return false;
                } else if (charCode == 13 && this.value.charCodeAt(this.selectionStart - 1) == 10 && this.value.charCodeAt(this.selectionStart - 2) == 10) {
                    if ($('.tour-form').length) {
                        showError(thisObj.next(), 'We\'re sorry, but that character is not allowed', false, 3000);
                    }
                    return false;
                } else if (charCode == 13 && this.value.charCodeAt(this.selectionStart - 1) == 10 && this.value.charCodeAt(this.selectionStart) == 10) {
                    if ($('.tour-form').length) {
                        showError(thisObj.next(), 'We\'re sorry, but that character is not allowed', false, 3000);
                    }
                    return false;
                } else if (charCode == 13 && this.value.charCodeAt(this.selectionStart) == 10 && this.value.charCodeAt(this.selectionStart + 1) == 10) {
                    if ($('.tour-form').length) {
                        showError(thisObj.next(), 'We\'re sorry, but that character is not allowed', false, 3000);
                    }
                    return false;
                }
            }
            var part1 = this.value.substring(0,this.selectionStart);
            var part2 = this.value.substring(this.selectionEnd,this.value.length);
            if (trim && (part1 == '' || part1.substring(part1.length - 1, part1.length) == ' ')) {
                if (String.fromCharCode(charCode) == ' ') {
                    if ($('.tour-form').length) {
                        showError(thisObj.next(), 'We\'re sorry, but that character is not allowed', false, 3000);
                    }
                    return false;
                }
            } else if (trim) {
                if (String.fromCharCode(charCode) == ' ' && this.value.charCodeAt(this.selectionStart) == 32) {
                    if ((this.selectionStart + 1) != this.value.length) {
                        // Put cursor one position ahead
                        var start = this.selectionStart + 1;
                        if (this.setSelectionRange) {
                            this.setSelectionRange(start, start);
                        }
                        if ($('.tour-form').length) {
                            showError(thisObj.next(), 'We\'re sorry, but that character is not allowed', false, 3000);
                        }
                        return false;
                    }
                }
            }

            if (charCode <= 40 && charCode != 32 && charCode != 33 && charCode != 34 && charCode != 35 && charCode != 36 && charCode != 37  && charCode != 38) {
                if (!mask.test(part1 + part2)) {
                    return false;
                }
            } else {
                if (!mask.test(part1 + String.fromCharCode(charCode) + part2)) {
                    if ($('.tour-form').length) {
                        showError(thisObj.next(), 'We\'re sorry, but that character is not allowed', false, 3000);
                    }
                    return false;
                }
            }
        }).bind("dragover", false)
            .bind("dragenter", false)
            .bind('drop', function(e) {
                this.value += e.originalEvent.dataTransfer.getData("text") || e.originalEvent.dataTransfer.getData("text/plain");
                var element = this;
                $(element).trigger('paste');
            }).bind('paste', function(e) {
            var element = this;
            setTimeout(function () {
                var text = $(element).val();
                if (mask == '/^[0-9]+$/') {
                    if (text.length >= 4 && (text.substring(text.length - 3, text.length - 2) == ',' || text.substring(text.length - 3, text.length - 2) == '.')) {
                        text = text.substring(0, text.length - 3);
                    }
                    text = text.replace(",", "").replace(".", "");
                    $(element).val(text);
                }
                if (!mask.test(text)) {
                    var mask_str = $(element).attr('data-mask');
                    if (mask_str.substring(0, 2) == '[^') {
                        var re = new RegExp("[" + mask_str.substring(2, mask_str.length - 2) + "]", "g");
                        text = text.replace(re, "");
                        $(element).val(text);
                    } else {
                        $(element).val('');
                        text = '';
                    }
                    $(element).trigger('keyup');
                }
                if (trim) {
                    text = text.replace(/ {2,}/g, " ");
                    $(element).val(text);
                }
                if (!block_enter) {
                    text = text.replace(/\n{3,}/g, "\n\n");
                    $(element).val(text);
                }

                text = $.trim(text.replace(/\u00a0/g, " "));
                $(element).val(text);

                $(element).trigger('keypress');
            }, 100);
        });
    };

    var inputs_mask = $("*[data-mask]");
    $(inputs_mask).each(function() {
        $(this).regexMask(new RegExp('^' + $(this).attr('data-mask') + '$'), $(this).attr('data-trim'), $(this).attr('data-enter'));
    });

    var inputs_min = $("*[data-min]");
    var inputs_max = $("*[data-max]");
    var inputs = $.unique( $.merge(inputs_min, inputs_max) );
    $(inputs).each(function() {
        var val_len = $(this).val().length;
        if($(this).data('excludehtml') == 1) {
            var val_len = $("<div/>").html($(this).val()).text().length;
        }
        if($(this).prev().hasClass("backdrop")) {
            if (!$(this).parent().next().hasClass('limit-msg')) {
                $(this).parent().after('<div class="limit-msg">You typed <b>' + val_len + '</b> characters and have <b>' + ($(this).attr('data-max') - val_len) + '</b> left (min. ' + $(this).attr('data-min') + ' max. ' + $(this).attr('data-max') + ' characters)</div>');
            } else {
                $(this).parent().next().html('You typed <b>' + val_len + '</b> characters and have <b>' + ($(this).attr('data-max') - val_len) + '</b> left (min. ' + $(this).attr('data-min') + ' max. ' + $(this).attr('data-max') + ' characters)');
            }

            if (val_len >= $(this).attr('data-min') && val_len <= $(this).attr('data-max')) {
                $(this).parent().next().addClass('within-range');
            } else {
                $(this).parent().next().removeClass('within-range');
            }
        } else {
            if (!$(this).next().hasClass('limit-msg')) {
                $(this).after('<div class="limit-msg">You typed <b>' + val_len + '</b> characters and have <b>' + ($(this).attr('data-max') - val_len) + '</b> left (min. ' + $(this).attr('data-min') + ' max. ' + $(this).attr('data-max') + ' characters)</div>');
            } else {
                $(this).next().html('You typed <b>' + val_len + '</b> characters and have <b>' + ($(this).attr('data-max') - val_len) + '</b> left (min. ' + $(this).attr('data-min') + ' max. ' + $(this).attr('data-max') + ' characters)');
            }

            if (val_len >= $(this).attr('data-min') && val_len <= $(this).attr('data-max')) {
                $(this).next().addClass('within-range');
            } else {
                $(this).next().removeClass('within-range');
            }
        }
    });
    $(inputs).bind('keyup', function() {
        var val_len = $(this).val().length;
        if($(this).data('excludehtml') == 1) {
            var val_len = $("<div/>").html($(this).val()).text().length;
        }
        if($(this).prev().hasClass("backdrop")) {
            $(this).parent().next().html('You typed <b>' + val_len + '</b> characters and have <b>' + ($(this).attr('data-max') - val_len) + '</b> left (min. ' + $(this).attr('data-min') + ' max. ' + $(this).attr('data-max') + ' characters)');
            if (val_len >= $(this).attr('data-min') && val_len <= $(this).attr('data-max')) {
                $(this).parent().next().addClass('within-range');
            } else {
                $(this).parent().next().removeClass('within-range');
            }
        } else {
            $(this).next().html('You typed <b>' + val_len + '</b> characters and have <b>' + ($(this).attr('data-max') - val_len) + '</b> left (min. ' + $(this).attr('data-min') + ' max. ' + $(this).attr('data-max') + ' characters)');
            if (val_len >= $(this).attr('data-min') && val_len <= $(this).attr('data-max')) {
                $(this).next().addClass('within-range');
            } else {
                $(this).next().removeClass('within-range');
            }
        }
    });
}


function jumpTo(id, correction) {
    if(id == "top") {
        pos = 0;
    } else {
        if (!$(id).length)
            return;
        
        if ($(".tour-form-jump").length) {
            var pos = $(id).offset().top - 60;
        } else {
            var pos = $(id).offset().top;
        }
        if (typeof id == 'string') {
            if (id == '#overall-errors' || id.indexOf("-title") !== -1) {
                pos -= 30;
            }
        }
        if (id == '#top') {
            pos = 0;
        }
    }

    if (typeof correction != 'undefined') {
        pos += correction;

        if (pos < 0) {
            pos = 0;
        }
    }

    $("html, body").animate({ scrollTop: pos }, 300);
    document.cookie = "jumpto=; expires=Thu, 01 Jan 1970 00:00:01 GMT;path=/";
}

function showError(after, text, solved_error, hideAfter) {
    if (typeof is_tour_form != 'undefined' && is_tour_form && typeof debugFunctionCall != 'undefined') {
        debugFunctionCall('showError');
    }
    var found = false;
    var foundElm = false;
    if (typeof solved_error != 'undefined') {
        if (solved_error !== false) {
            removeSolved(solved_error);
        }
    }
    if ($(after).next().is('ul.error')) {
        $(after).next().find('li').each(function() {
            var compareText = $("<div />").html(text).html();
            if ($(this).html() == compareText) {
                found = true;
                foundElm = $(this);
            }
            if (found == false && typeof solved_error != 'undefined' && solved_error) {
                if ($(this).hasClass(solved_error)) {
                    found = true;
                    foundElm = $(this);
                }
            }
        });
    } else {
        $(after).after('<ul class="error"></ul>');
    }
    if (!found) {
        if (typeof solved_error != 'undefined' && solved_error) {
            $(after).next().append('<li class="adding ' + solved_error + '">' + text + '</li>');
        } else {
            $(after).next().append('<li class="adding">' + text + '</li>');
        }
        foundElm = $(after).next().find('li').last();
        $(after).next().find('li').last().slideDown('normal', function(){
            $(this).removeClass('adding');
            $(this).animate({'opacity': '0.3'}, 350).animate({'opacity': '1.0'}, 350).animate({'opacity': '0.3'}, 350).animate({'opacity': '1.0'});
        });
        if (typeof hideAfter != 'undefined') {
            if ($(after).data('timer')) {
                clearTimeout($(after).data('timer'));
            }
            var timer = setTimeout(function() {
                var element = $(foundElm);
                $(element).addClass('removing');
                $(element).slideUp('normal', function() {
                    var li_cnt = $(element).parent().find('li').length;
                    var parent_elm = $(element).parent();
                    $(element).remove();
                    if (li_cnt == 1) {
                        $(parent_elm).remove();
                    }
                });
            }, hideAfter);
            $(after).attr('data-timer', timer);
        }
    } else {
        if ($(after).data('timer')) {
            clearTimeout($(after).data('timer'));
        }
        if (typeof hideAfter != 'undefined') {
            $(foundElm).animate({'opacity': '0.3'}, 350).animate({'opacity': '1.0'}, 350).animate({'opacity': '0.3'}, 350).animate({'opacity': '1.0'});
            var timer = setTimeout(function() {
                var element = $(foundElm);
                $(element).addClass('removing');
                $(element).slideUp('normal', function() {
                    var li_cnt = $(element).parent().find('li').length;
                    var parent_elm = $(element).parent();
                    $(element).remove();
                    if (li_cnt == 1) {
                        $(parent_elm).remove();
                    }
                });
            }, hideAfter);
            $(after).attr('data-timer', timer);
        }
    }

    if ($(after).closest('.error-section').length) {
        $(after).closest('.error-section').find('.section-title').addClass('error');
    }
}

function setSolved(solved_error) {
    if (typeof is_tour_form != 'undefined' && is_tour_form && typeof debugFunctionCall != 'undefined') {
        debugFunctionCall('setSolved');
    }
    try {
        if ($(".error-" + solved_error).length && !$(".error-" + solved_error).hasClass('solved')) {
            $(".error-" + solved_error).addClass('solved');
            if (typeof handleErrors == 'function') {
                handleErrors();
            }
        }
    } catch(err) { }
}

function removeSolved(solved_error) {
    if (typeof is_tour_form != 'undefined' && is_tour_form && typeof debugFunctionCall != 'undefined') {
        debugFunctionCall('removeSolved');
    }
    try {
        if ($(".error-" + solved_error).length && $(".error-" + solved_error).hasClass('solved')) {
            $(".error-" + solved_error).removeClass('solved');
            if ($(".error-" + solved_error).parent().parent().hasClass('solved')) {
                $(".error-" + solved_error).parent().parent().removeClass('solved');
                $(".error-" + solved_error).parent().parent().find(".solved-icon").remove();
            }
            $(".error-" + solved_error).find('.solved-icon').remove();
            if (typeof handleErrors == 'function') {
                handleErrors();
            }
        }
    } catch(err) { }
}

var onZeroErrors = null;

function hideError(after, text, solved_error, next) {

    if (typeof is_tour_form != 'undefined' && is_tour_form && typeof debugFunctionCall != 'undefined') {
        debugFunctionCall('hideError');
    }

    if (typeof solved_error != 'undefined') {
        if (solved_error !== false) {
            setSolved(solved_error);
        }
    }
    if ($(after).next().is('ul.error')) {
        $(after).next().find('li').each(function() {
            var compareText = $("<div />").html(text).html();
            if ($(this).html() == compareText || compareText == '' || (typeof solved_error != 'undefined' && solved_error && $(this).hasClass(solved_error))) {
                var element = $(this);
                $(element).addClass('removing');
                if($(element).is(":visible")) {
                    $(element).slideUp('normal', function() {
                        var li_cnt = $(element).parent().find('li').length;
                        var parent_elm = $(element).parent();
                        $(element).remove();
                        if (li_cnt == 1) {
                            $(parent_elm).remove();
                        }
                        if ($(after).closest('.error-section').length && !$(after).find("ul.error").length) {
                            $(after).closest('.error-section').find('.section-title').removeClass('error');
                        }

                        if ($("ul.error").length == 0 && onZeroErrors!=null)
                            onZeroErrors();

                        if (next)
                            next();

                    });
                }
            }
        });
    }

}

function showHideError(condition, after, text, solved_error_or_object) {
    if (typeof is_tour_form != 'undefined' && is_tour_form && typeof debugFunctionCall != 'undefined') {
        debugFunctionCall('showHideError');
    }

    if (typeof solved_error_or_object == 'undefined') {
        solved_error_or_object = false;
    } else {
        if (typeof solved_error_or_object == 'object') {
            if (condition) {
                solved_error_or_object.addClass('error');
            } else {
                solved_error_or_object.removeClass('error');
            }
            solved_error_or_object = false;
        }
    }
    if (condition) {
        showError(after, text, solved_error_or_object);
    } else {
        hideError(after, text, solved_error_or_object);
    }
}

function addSaveLayer() {
    $("body").append('<div class="save-layer"><div><img src="/images/loader.gif" /> Processing, please wait...</div></div>');
    $(".save-layer div").css('margin-top', ($(window).height() / 2) - ($(".save-layer div").height() / 2));
    $(".save-layer").fadeIn('slow');
}

function removeSaveLayer() {
    $(".save-layer").stop().fadeOut();
}

function closeSB() {
    window.parent.Shadowbox.close();
    return false;
};

function initHelpMarks() {
    window.setTimeout(function() {
        $(".help").balloon();
    }, 10);
}

function getDirection(mouseposition) {
    var myHeight = 0;
    if( typeof( window.innerWidth ) == 'number' ) {
        //Non-IE

        myHeight = window.innerHeight;
    } else if( document.documentElement && ( document.documentElement.clientWidth || document.documentElement.clientHeight ) ) {
        //IE 6+ in 'standards compliant mode'

        myHeight = document.documentElement.clientHeight;
    } else if( document.body && ( document.body.clientWidth || document.body.clientHeight ) ) {
        //IE 4 compatible

        myHeight = document.body.clientHeight;
    }

    return (myHeight - mouseposition);
}

function getXDirection(mouseposition) {
    var myWidth = 0;
    if( typeof( window.innerWidth ) == 'number' ) {
        //Non-IE

        myWidth = window.innerWidth;
    } else if( document.documentElement && ( document.documentElement.clientWidth || document.documentElement.clientHeight ) ) {
        //IE 6+ in 'standards compliant mode'

        myWidth = document.documentElement.clientWidth;
    } else if( document.body && ( document.body.clientWidth || document.body.clientHeight ) ) {
        //IE 4 compatible

        myWidth = document.body.clientWidth;
    }

    return (myWidth - mouseposition);
}

//----------------------------------
// opens a shadow box
//TODO: fix title, it is not shown but is there in HMTL
function openShadowbox(width, height, url, title){
    $("#sb-body").css('background-color','#ffffff');
    $("#sb-loading").css('background-color','#ffffff');
    $("#sb-loading-inner > span").css('background-image','url(/images/load.gif)');
    Shadowbox.open({
        player: 'iframe',
        content: url,
        height: height,
        width: width,
        title: 'Press <b>ESC</b> to return'
    });
}

//----------------------------------
// closes a shadowbox
function closeShadowbox()
{
    if (window.parent.Shadowbox!=null)
        window.parent.Shadowbox.close();
}

//----------------------------------
// gets a field value from an xml structure
function xmlValue(xml,field)
{
    try {
        return xml.getElementsByTagName(field)[0].firstChild.nodeValue;
    }
    catch(e) {
        return "";
    }
}

//----------------------------------
// Creates an data:Object from form field values
// param is dataVars[]: an array of string (== form fieldnames)
//----------------------------------
// each form  fieldname has the format : id(|element selector)
// and is looked up using :
// object[id] = $("#{id}").val();
// .. or ...
// object[id] = $("#{element selector}").val();
//----------------------------------
function createFormData(dataVars)
{
    var obj = new Object();
    for (var i=0; i<dataVars.length; i++)
    {
        var v = dataVars[i];
        if (v.indexOf("|")>=0) {
            var va = v.split("|");
            obj[va[0]]= $(va[1]).val();
        }
        else
            obj[v] = $("#"+v).val();
    }
    return obj;
}

//----------------------------------
// general post method that returns xml and execute an onsuccess or on error handler
// ------- params ------------------
// Object data : post variabelen
// string url : post url
// function onSuccess(xmlData);
// function onError(xmlData, errorMessage);
//------- remarks ------------------
// xmlData bevat de xml, waar via xmlValue een waarde uit kan worden gehaald.
//----------------------------------
function postFormXml(data, url, onSuccess, onError)
{
    $.post(url, data,
        function(data) {
            var status = xmlValue(data,"status");
            var error = xmlValue(data,"error")
            if (status=="error" || status=="err" || error == "1"  || error == "true") {
                if (onError) {
                    var msg = xmlValue(data,"statusMsg");
                    if (msg=="") msg = xmlValue(data,"errorMsg");
                    if (msg=="") msg = xmlValue(data,"errorString");
                    if (msg=="") msg = xmlValue(data,"errorMessage");
                    if (msg=="") msg = xmlValue(data,"message");
                    onError(data, msg);
                }
            }
            else {
                if (onSuccess) onSuccess(data);
            }
        }, "xml"
    ).fail(function(xhr, msg, err){
        err = ""+err;
        if (err.toLowerCase() == "not found")
            msg = "Destination not found!";
        if (onError)
            onError({ xhr:xhr, msg : msg, error : err} ,msg);
    });
}

function getDirectionMain(mouseposition) {
    var myHeight = 0;
    if( typeof( window.innerWidth ) == 'number' ) {
        //Non-IE

        myHeight = window.innerHeight;
    } else if( document.documentElement && ( document.documentElement.clientWidth || document.documentElement.clientHeight ) ) {
        //IE 6+ in 'standards compliant mode'

        myHeight = document.documentElement.clientHeight;
    } else if( document.body && ( document.body.clientWidth || document.body.clientHeight ) ) {
        //IE 4 compatible

        myHeight = document.body.clientHeight;
    }

    return (myHeight - mouseposition);
}

function getXDirectionMain(mouseposition) {
    var myWidth = 0;
    if( typeof( window.innerWidth ) == 'number' ) {
        //Non-IE

        myWidth = window.innerWidth;
    } else if( document.documentElement && ( document.documentElement.clientWidth || document.documentElement.clientHeight ) ) {
        //IE 6+ in 'standards compliant mode'

        myWidth = document.documentElement.clientWidth;
    } else if( document.body && ( document.body.clientWidth || document.body.clientHeight ) ) {
        //IE 4 compatible

        myWidth = document.body.clientWidth;
    }

    return (myWidth - mouseposition);
}

function cookieEnvName(name) {
    var env = 'production';
    if ($('html').hasClass('development')) {
        env = 'development';
    }
    else if ($('html').hasClass('demo')) {
        env = 'demo';
    }
    else if ($('html').hasClass('staging')) {
        env = 'staging';
    }

    if (env == 'demo') {
        name = 'dm_' + name;
    }
    else
    if (env != 'production') {
        name = env.substring(0, 1) + '_' + name;
    }
    return name;
}

function getCookie(name) {
    var iname = cookieEnvName(name) + "=";
    var ca = document.cookie.split(';');
    for(var i = 0; i < ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0) == ' ') {
            c = c.substring(1);
        }
        if (c.indexOf(iname) == 0) {
            var str = c.substring(iname.length, c.length).replace(/\+/g, ' ');
            return decodeURIComponent(str);
        }
    }

    return "";
}

function setCookie(name, value, days) {

    var expires = "";
    if(typeof days !== "undefined") {
        var d = new Date();
        d.setTime(d.getTime() + (days*24*60*60*1000));
        expires += ";expires="+ d.toUTCString();
    }

    var cookie_val = encodeURIComponent(value);

    cookie_val = cookie_val.replace(/%20/g, "+");

    if (typeof cookie_domain == 'undefined') {
        cookie_domain = 'AfricaGoTrips.com'
    }

    document.cookie = cookieEnvName(name) + "=" + cookie_val + ";domain=." + cookie_domain + expires + ";path=/"+((location.protocol=="https:") ? ";secure" : "");

}

$helper = {
    "uncacheAjaxCalls" : function(url) {
        var uts = new Date().getTime();
        if (url.indexOf('?') !== -1) {
            url += "&uts=" + uts;
        } else {
            url += "?uts=" + uts;
        }
        return url;
    },
    "eventHandler" : function (el,ev, f) {
        $(el).off(ev);
        $(el).on(ev, f);
    },
    "ajaxCall" : function(url, pData, onSuccess, dataType, opener) {

        opener = (typeof opener === 'undefined') ? false : opener;

        if (opener)
        {
            if (typeof opener.before !== 'undefined')
                opener.before.call(opener.element);
        }

        var self = this;
        if (dataType == null)
            dataType = "text";
        $.ajax({
            data : pData,
            url: this.uncacheAjaxCalls(url),
            success: function(data) {
                if (onSuccess!=null)
                {
                    onSuccess(data);
                    if (opener)
                    {
                        if (typeof opener.after !== 'undefined')
                            opener.after.call(opener.element);
                    }
                }
            },
            statusCode: {
                404: function() {
                    if (opener)
                    {
                        if (typeof opener.after !== 'undefined')
                            opener.after.call(opener.element);
                    }
                }
            },
            dataType: dataType
        });
    },
    "jsonCall" : function(url, pData, onSuccess, opener) {
        opener = (typeof opener === 'undefined') ? false : opener;

        var self = this;
        this.ajaxCall(url,pData,onSuccess,"json", opener);
    }
};


function initLoggedHeader() {

    // we put jQuery selectors into vars for re-use (optimalisation)
    var $pansignasoperator = $("#pansignasoperator");
    var $signinsignaslink = $("#signinsignaslink");
    var $signoutsignaslink = $("#signoutsignaslink");

    var signInAs = function(operator_id) {

        $sb.jsonCall("/account/sign-in-as", {operator_id : operator_id}, function(data) {
            window.setTimeout(function() {
                if (data.selected_operator_id > 1)
                    window.location.assign("/dashboard?_op="+data.selected_operator_id);
                else
                    window.location.assign("/dashboard");
            }, 250);
        });

    };

    fPan = function() {

        var operator_id = $sb._op;

        var changeOperatorId = function($op) {

            var isSimulated = $pansignasoperator.attr("data-admin") === "1";

            operator_id = $op;
            if (operator_id === 0)
                operator_id = 1;

            $signinsignaslink.hide();
            if (!isSimulated) {
                if (operator_id <= 1)
                    return false;
            }

            signInAs(operator_id);
        };

        var isSimulated = $pansignasoperator.attr("data-admin") === "1";

        if ($pansignasoperator.is(":visible")) {
            $pansignasoperator.hide();
        }
        else {

            $sb.signinAs = function($op) {
                changeOperatorId($op);
            };

            $pansignasoperator.show();
            $signinsignaslink.hide();

            // show sign in link if we are not simulating and enter popup with a selected operator
            // when we come in with AfricaGoTrips, a 'change' select will be the go-ahead action
            if (!isSimulated) {
                if (operator_id > 1)
                    $signinsignaslink.show();
            }

            $helper.eventHandler("#signinsignaslink","click", function() {
                signInAs(operator_id);
                return false;
            });

            $helper.eventHandler("#cancelsignaslink","click", function() {
                $pansignasoperator.hide();
                return false;
            });

            $helper.eventHandler("#signoutsignaslink","click", function() {
                signInAs(1);
                return false;
            });

        }
        return false;
    };

    $helper.eventHandler("#linksignasoperator","click", fPan);
    $helper.eventHandler("#linksignoffoperator","click", fPan);

}

function initLimitProgress() {
    if ($(".billing-limits").length && $(".billing-limits").is(":visible")) {
        $(".billing-limits .bar").css('width', '100%');
        $(".billing-limits span").css('right', '0');
        $(".billing-limits em").css('right', '7px');
        var budget_perc = $(".billing-limits .holder").data('budget-perc');
        var days_perc = $(".billing-limits .holder").data('days-perc');

        if (budget_perc < days_perc && budget_perc <= 25) {
            $(".billing-limits .limit-bar").addClass('red');
        } else if (budget_perc < days_perc && budget_perc < 50) {
            $(".billing-limits .limit-bar").addClass('orange');
        }

        if (budget_perc < 100) {
            $(".billing-limits .bar").stop().delay(1000).animate({'width': budget_perc + '%'}, 1000);
        }
        if (days_perc < 100) {
            var daysleft_right = Math.round($(".billing-limits .bar").outerWidth() - (($(".billing-limits .bar").outerWidth() / 100) * days_perc) - ($(".billing-limits span").outerWidth() / 2));
            var daysleft_pointer_right = Math.round($(".billing-limits .bar").outerWidth() - (($(".billing-limits .bar").outerWidth() / 100) * days_perc));
            var max_daysleft_right = $(".billing-limits .holder").outerWidth() - ($(".billing-limits span").outerWidth() + 2);
            if (daysleft_right > max_daysleft_right) {
                daysleft_right = max_daysleft_right;
            } else if (daysleft_right < 0) {
                daysleft_right = 0;
            }
            if (daysleft_pointer_right > $(".billing-limits .holder").outerWidth() - 10) {
                daysleft_pointer_right = $(".billing-limits .holder").outerWidth() - 10;
            } else if (daysleft_pointer_right < 7) {
                daysleft_pointer_right = 7;
            }
            $(".billing-limits span").stop().delay(1000).animate({'right': daysleft_right + 'px'}, 1000);
            $(".billing-limits em").stop().delay(1000).animate({'right': daysleft_pointer_right + 'px'}, 1000);
        }
    }
}

function handleOpenRevealModal() {
    $(".reveal-modal-bg").fadeIn(400, function() {
        if (!$("#surface").hasClass('modal-open')) {
            var scroll_top = $(window).scrollTop();
            $("#surface").addClass("modal-open");
            if (scroll_top > 0) {
                $("#surface").css('top', (scroll_top * -1));
                $(".reveal-modal.open").css('top', '-=' + (scroll_top) + 'px');

                $(window).scrollTop(0);
            }
        }
    });
    removeSBLoadLayer();
    if (!$(".reveal-modal.open").find('.close-reveal-modal').length) {
        $(".reveal-modal.open").append('<a class="close-reveal-modal" href="#">Close</a>');
    }
    initHelpMarks();
}

function handleCloseRevealModal() {
    if ($("#surface").hasClass('modal-open')) {
        $("#surface").removeClass("modal-open");
        document.body.style.overflow = "auto";
        var current_top = parseInt($("#surface").css('top'));
        if (current_top && current_top < 0) {
            $("#surface").css('top', 0);
            $(window).scrollTop(current_top * -1);
        }
    }
}

function findGetParameter(parameterName) {
    var result = null,
        tmp = [];
    var items = location.search.substr(1).split("&");
    for (var index = 0; index < items.length; index++) {
        tmp = items[index].split("=");
        if (tmp[0] === parameterName) result = decodeURIComponent(tmp[1]);
    }
    return result;
}

$(document).ready(function(){

    $(window).on('resize', function() {

        if ($(".email-box-holder").length)
            emailBoxDimensions();

        if ($(".history-box-holder").length)
            historyBoxDimensions();

        if ($(".problem-box-holder").length)
            problemBoxDimensions();

    });

    if (!$(".requests-table").length && !$(".tour-table").length) {
        // Fixed problem that Help marks on requests-table & tour-table page are initialized to early (in combination with tableSorter
        initHelpMarks();
    }
    initLoggedHeader();

    $('.tours-table').on('click', 'td', function(e) {
        if(!$(this).closest("table").hasClass("acco-table") || !$(e.target).hasClass("complete")) {
            $(this).parent().find('a').last().trigger('click');
        }
    });

    $('.tours-table').on('click', '.edit', function(e) {
        e.preventDefault();
        e.stopPropagation();

        if ($(this).attr("href").indexOf("/company/edit") == -1 && $(this).attr('data-info') && $(this).attr('data-info') == 1 && $(this).attr('data-val') && $(this).attr('data-val') == 2) {
            window.location = $(this).attr("href");
            window.open($(this).attr("href").replace('/edit/', '/info/'), 'Check info page', 'resizable=1, scrollbars=1, fullscreen=1, screenX=0 , left=1280, toolbar=0, menubar=0, status=1');
        } else if ($(this).attr("href").indexOf("/content/accommodation") != -1) {
            window.location = $(this).attr("href");
            if($(this).is('[data-url]')) {
                window.open($(this).attr("data-url"), 'Check info page', 'resizable=1, scrollbars=1, fullscreen=1, screenX=0 , left=1280, toolbar=0, menubar=0, status=1');
            }
        }

        
    });
});

// Reveal modal helper functions for smoothly opening and closing the modals

$(document).on('open.fndtn.reveal', '[data-reveal]', function () {
    if (!$('#reveal_modal').find("#view-new-operator").length) {
        $("body").css({
            'overflow': 'hidden'
        });
    }
});
$(document).on('closed.fndtn.reveal', '[data-reveal]', function () {
    $("body").css({
        'overflow': 'visible'
    })
});

function strWordCount(string) {

    var words = string.replace(/([(\\\.\+\*\?\[\^\]\$\(\)\{\}\=\!<>\|\:])/g, '');
    words = words.replace(/(^\s*)|(\s*$)/gi,"");
    words = words.replace(/[ ]{2,}/gi," ");
    words = words.replace(/\n /,"\n");
    words = words.replace(/[0-9]/gi,"");

    if(words == '' || words == ' ') {
        return 0;
    }

    return words.split(' ').length;

}

function numberformat (number, decimals, dec_point, thousands_sep) {
    number = (number + '').replace(/[^0-9+\-Ee.]/g, '');
    var n = !isFinite(+number) ? 0 : +number,
        prec = !isFinite(+decimals) ? 0 : Math.abs(decimals),
        sep = (typeof thousands_sep === 'undefined') ? ',' : thousands_sep,
        dec = (typeof dec_point === 'undefined') ? '.' : dec_point,
        s = '',
        toFixedFix = function (n, prec) {
            var k = Math.pow(10, prec);
            return '' + Math.round(n * k) / k;
        };
    // Fix for IE parseFloat(0.55).toFixed(0) = 0;
    s = (prec ? toFixedFix(n, prec) : '' + Math.round(n)).split('.');
    if (s[0].length > 3) {
        s[0] = s[0].replace(/\B(?=(?:\d{3})+(?!\d))/g, sep);
    }
    if ((s[1] || '').length < prec) {
        s[1] = s[1] || '';
        s[1] += new Array(prec - s[1].length + 1).join('0');
    }
    return s.join(dec);
}

$(document).on('submit', 'form[data-role=input-successrate]', function(e){
    e.preventDefault();
    var successrate_output;

    $.ajax({
        type: "POST",
        url: "/admin/successrate",
        data: {
            oid: $("#success-rate-oid").val()
        }
    }).done(function(data) {
        var rep;
        if(data == 1) {
            rep = 'successful';
        } else {
            rep = 'unsuccessful';
        }
        successrate_output = 'Tour operator: <strong>' + $("#success-rate-oid").val() + '</strong> is <strong>' + rep + '</strong>';

    }).fail(function() {
        successrate_output = 'something went wrong! Try again.';
    }).always(function() {
        $('#success-rate-submit').removeClass('loading');
        $('[data-role=output-successrate]').html(successrate_output);
    });
});


function escapeHtml(text) {
    var map = {
        '&': '&amp;',
        '<': '&lt;',
        '>': '&gt;',
        '"': '&quot;',
        "'": '&#039;'
    };

    return text.replace(/[&<>"']/g, function(m) { return map[m]; });
}

function close_operator_open_dropdown() {

    var $chevron = $(".area-operatorselect").find(".chevron");
    var $drop = $('ul#operator-open-options-dropdown');
    if ($chevron && $drop && $chevron.length && $drop.length) {
        // handle all chosen on this response
        if ($drop.is(":visible")) {
            $drop.remove();
            $chevron.removeClass("active");
        }
    }
}

function edit_company_profile(hash) {

    if (hash === undefined) {
        hash = "";
    }
    if ($sb._op!=null) {
        close_operator_open_dropdown()
        window.setTimeout(function() {
            window.location.href = $sb.backend_url("company/edit") + hash;
        },250);
    }
}

function adminactions(hash) {

    if (hash === undefined) {
        hash = "";
    }
    if ($sb._op!=null) {
        close_operator_open_dropdown()
        window.setTimeout(function() {
            window.location.href = $sb.backend_url("companies/operator") + hash;
        },250);
    }
}

function edit_company_profile_contacts() {
    edit_company_profile("#contacts");
}


$(function() {

    $chosenAuto = $(".chosen.auto");
    if ($chosenAuto.length && $chosenAuto.chosen) {
        $chosenAuto.chosen({
            disable_search_threshold: 10,
            no_results_text: "Oops, nothing found!"
        });
    }

    $(document).off("click","#operators-select-open");
    $(document).on("click","#operators-select-open", function() {
        adminactions();
    });

    $(document).off("click","#operators-select-undo");
    $(document).on("click","#operators-select-undo", function() {
        if (window.location.href.indexOf($sb.url_root+"/companies") === 0) {
            $sb._op = 1;
            window.location.assign($sb.backend_url("companies/onboarding"));
        }
        else{
            var inBillingInvoices = window.location.href.indexOf("/billing/invoices")>=0;
            window.location.assign($sb.windowLocation({
                "_op": 1
            },(inBillingInvoices && $sb._op>1)?["pt","sd","ed"]:[] ));
        }

    });

    $(document).off("click",".area-operatorselect .chevron");
    $(document).on("click",".area-operatorselect .chevron", function() {

        var chevron = $(this).parents(".area-operatorselect").find(".chevron");
        var drop = $("#operator-open-options-dropdown");

        if(chevron.hasClass("active")) {

            chevron.removeClass("active");
            drop.remove();

        } else {

            drop.remove();
            chevron.removeClass("active");

            var items = "<li><a href='' data-val='onboarding' class='jump-to-edit'>Onboarding</a></li>";
            if ($sb._op != null) {

                var al = chevron.data("al");
                items = "<li><a href='javascript:adminactions();' data-val='admin-actions' class='jump-to-edit'>Admin Actions</a></li>";

                if (al === "full")
                    items += "<li><a href='"+$sb.backend_url("billing/billingprofile")+"' data-val='billing-actions' class='jump-to-edit'>Billing Profile</a></li>";

                if ($sb.signup_id!=null)
                    items+="<li><a href='"+$sb.backend_url("companies?ob=1")+"' data-val='onboarding' class='jump-to-edit'>Onboarding</a></li>";
                else
                    items+="<li class='disabled'><a title='The selected operator has no onboarding data' href='javascript:;' data-val='onboarding' class='jump-to-edit'>Onboarding</a></li>";

                if (al ==='full' || al === 'crm')
                    items+="<li><a href='"+$sb.backend_url("admin/requests?q=p"+$sb._op)+"' data-val='requests' class='jump-to-edit'>Requests <small>(Admin view)</small></a></li>";

                if (al!=='crm' && al!=='content')
                    items+="<li><a href='"+$sb.backend_url( "content/reviews/search/p"+$sb._op)+"' data-val='reviews' class='jump-to-edit'>Reviews <small>(Admin view)</small></a></li>";

                items+="<li><a href='javascript:edit_company_profile();' data-val='edit-profile' class='jump-to-edit'>Edit Profile</a></li>";
                items+="<li><a href='"+$sb.front_root+"/p"+$sb._op+"' target='_blank' data-val='view-profile' class='jump-to-edit'  rel='noopener'>View Profile</a></li>";
                items+="<li><a href='javascript:edit_company_profile_contacts();' data-val='view-contacts' class='jump-to-edit'>View Contacts</a></li>";
            }

            $(this).parents(".area-operatorselect").find(".operator-select-box").before("<ul id='operator-open-options-dropdown'>" + items + "</ul>");

            drop = $("#operator-open-options-dropdown");
            var chevron_right = chevron.offset().left + chevron.outerWidth();
            var chevron_top = chevron.offset().top;
            var chevron_height = chevron.outerHeight();
            var drop_right = drop.offset().left + drop.outerWidth();
            var drop_top = drop.offset().top;

            var drop_width = 153;

            drop.css({
                left: (chevron_right - drop_width - 10) + "px",
                marginTop:  0 + (chevron_top - drop_top) + chevron_height + 2 + "px",
                width: drop_width+"px"
            });
            chevron.addClass("active");

            $('body').click(function (e) {
                if ((!$(e.target).parents('#operator-open-options-dropdown').length && !$(e.target).is('#edit-options-dropdown') && !$(e.target).hasClass('chevron')) ) {
                    close_operator_open_dropdown();
                }
            });
        }
        return false;
    });


});